from typing import Literal

from pydantic import Field

from ._base_config import BasePropertyConfig
from ....models.primitives import EmptyObject
from ....properties.base_properties._base_property import NotionPropertyType


class PlacePropertyConfig(BasePropertyConfig[Literal[NotionPropertyType.PLACE]]):
    """Notionのplaceプロパティ設定

    TypeScript: PlacePropertyConfigurationRequest
    フィールドは空オブジェクトで構成されます。
    """

    type: Literal[NotionPropertyType.PLACE] = Field(
        NotionPropertyType.PLACE, description="プロパティタイプ"
    )
    place: EmptyObject = Field(
        default_factory=EmptyObject, description="place設定"
    )




from typing import Literal

from pydantic import Field

from ._base_config import BasePropertyConfig
from ....models.primitives import EmptyObject
from ....properties.base_properties._base_property import NotionPropertyType


class TitlePropertyConfig(BasePropertyConfig[Literal[NotionPropertyType.TITLE]]):
    """Notionのtitleプロパティ設定"""

    type: Literal[NotionPropertyType.TITLE] = Field(
        NotionPropertyType.TITLE, description="プロパティタイプ"
    )
    title: EmptyObject = Field(
        default_factory=EmptyObject, description="title設定"
    )



from pydantic import BaseModel, Field, StrictInt, StrictStr


class UniqueId(BaseModel):
    """Notionのunique_id値"""

    prefix: StrictStr | None = Field(None, description="ユニークIDのプレフィックス")
    number: StrictInt | None = Field(None, description="ユニークIDの番号")



from typing import Literal

from pydantic import BaseModel, Field, StrictStr

from ._base_config import BasePropertyConfig
from ....properties.base_properties._base_property import NotionPropertyType


class UniqueIdOptions(BaseModel):
    """unique_idプロパティの設定"""

    prefix: StrictStr | None = Field(None, description="ユニークIDのプレフィックス")


class UniqueIdPropertyConfig(
    BasePropertyConfig[Literal[NotionPropertyType.UNIQUE_ID]]
):
    """Notionのunique_idプロパティ設定"""

    type: Literal[NotionPropertyType.UNIQUE_ID] = Field(
        NotionPropertyType.UNIQUE_ID, description="プロパティタイプ"
    )
    unique_id: UniqueIdOptions = Field(
        default_factory=lambda: UniqueIdOptions(prefix=None),
        description="unique_id設定",
    )



from typing import Literal

from pydantic import Field

from ._base_config import BasePropertyConfig
from ....models.primitives import EmptyObject
from ....properties.base_properties._base_property import NotionPropertyType


class ButtonPropertyConfig(BasePropertyConfig[Literal[NotionPropertyType.BUTTON]]):
    """Notionのbuttonプロパティ設定"""

    type: Literal[NotionPropertyType.BUTTON] = Field(
        NotionPropertyType.BUTTON, description="プロパティタイプ"
    )
    button: EmptyObject = Field(default_factory=EmptyObject, description="button設定")



from pydantic import Field

from ...models import DateInfo
from ._base_property import BaseProperty, NotionPropertyType
from typing import Literal
from datetime import date as dt_date


class DateProperty(BaseProperty[Literal[NotionPropertyType.DATE]]):
    """Notionのdateプロパティ"""

    type: Literal[NotionPropertyType.DATE] = Field(
        NotionPropertyType.DATE, description="プロパティタイプ"
    )
    date: DateInfo | None = Field(
        None, description="日付情報（設定されていない場合はnull）"
    )

    def get_display_value(self) -> str | int | float | bool | None:
        """日付情報を取得

        Returns:
            str | None: 日付範囲を "開始日→終了日" の形式で返す。終了日がない場合は開始日のみを返す。未設定の場合はNone
        """
        if self.date is None:
            return None
        if self.date.end:
            return f"{self.date.start}→{self.date.end}"
        return self.date.start

    def get_start_date(self) -> dt_date:
        """
        date プロパティから開始日を date 型で取得

        Returns:
            date: 開始日（date型）
        Raises:
            ValueError: dateプロパティが未設定の場合
        Note:
            - 日付範囲が設定されている場合も開始日のみを返します
            - 終了日が必要な場合は、date.endを直接参照してください

        Examples:
            - 単一日付: date(2024, 3, 15)
            - 日付範囲: date(2024, 3, 15) (開始日のみ)
            - 未設定: None
        """
        if not self.date:
            raise ValueError("Date property is not set")
        return dt_date.fromisoformat(self.date.start)

    def get_end_date(self) -> dt_date:
        """
        date プロパティから終了日を date 型で取得

        Returns:
            date: 終了日（date型）
        Raises:
            ValueError: 終了日が設定されていない場合
        Note:
            - 単一日付の場合、終了日はNoneとなります
            - 終了日が必要な場合は、date.endを直接参照してください

        Examples:
            - 単一日付: None
            - 日付範囲: date(2024, 3, 20) (終了日のみ)
            - 未設定: None
        """
        if not self.date or not self.date.end:
            raise ValueError("End date is not set")
        return dt_date.fromisoformat(self.date.end)



from typing import Literal
from pydantic import Field

from ...models import SelectOption
from ._base_property import BaseProperty, NotionPropertyType


class MultiSelectProperty(BaseProperty[Literal[NotionPropertyType.MULTI_SELECT]]):
    """Notionのmulti_selectプロパティ"""

    type: Literal[NotionPropertyType.MULTI_SELECT] = Field(
        NotionPropertyType.MULTI_SELECT, description="プロパティタイプ"
    )

    multi_select: list[SelectOption] = Field(
        default_factory=list, description="選択されたオプション配列"
    )

    def get_display_value(self) -> str | int | float | bool | None:
        """選択されたオプション名のリストを取得

        Returns:
            str | None: 選択されたオプション名をカンマ区切りで連結した文字列。選択がない場合はNone
        """
        if len(self.multi_select) == 0:
            return None
        return ", ".join(option.name for option in self.multi_select)



from typing import Any, Generic, TypeVar

from pydantic import BaseModel, ConfigDict, Field, StrictStr

from ....properties.base_properties._base_property import NotionPropertyType

_TPropertyType = TypeVar("_TPropertyType", bound=NotionPropertyType)


class BasePropertyConfig(BaseModel, Generic[_TPropertyType]):
    """Notionのプロパティ設定のベースクラス"""

    id: StrictStr = Field(..., description="プロパティID")
    name: StrictStr = Field(..., description="プロパティ名")
    type: _TPropertyType
    description: StrictStr | None = Field(None, description="プロパティの説明")

    def to_dict(self) -> dict[str, Any]:
        """Notion APIに渡す辞書形式の設定"""
        return self.model_dump(exclude_none=True)

    def to_json(self) -> str:
        """JSON文字列にシリアライズ"""
        return self.model_dump_json(exclude_none=True)



"""
Notionブロックの基底クラスと共通モデル
"""

from __future__ import annotations

from enum import Enum
from typing import Literal

from pydantic import BaseModel, Field, StrictBool, StrictStr

from ..models.user import PartialUser
from ..models.parent import NotionParent


class BlockType(str, Enum):
    """ブロックタイプ"""

    PARAGRAPH = "paragraph"
    HEADING_1 = "heading_1"
    HEADING_2 = "heading_2"
    HEADING_3 = "heading_3"
    BULLETED_LIST_ITEM = "bulleted_list_item"
    NUMBERED_LIST_ITEM = "numbered_list_item"
    QUOTE = "quote"
    TO_DO = "to_do"
    TOGGLE = "toggle"
    TEMPLATE = "template"
    SYNCED_BLOCK = "synced_block"
    CHILD_PAGE = "child_page"
    CHILD_DATABASE = "child_database"
    EQUATION = "equation"
    CODE = "code"
    CALLOUT = "callout"
    DIVIDER = "divider"
    BREADCRUMB = "breadcrumb"
    TABLE_OF_CONTENTS = "table_of_contents"
    COLUMN_LIST = "column_list"
    COLUMN = "column"
    LINK_TO_PAGE = "link_to_page"
    TABLE = "table"
    TABLE_ROW = "table_row"
    EMBED = "embed"
    BOOKMARK = "bookmark"
    IMAGE = "image"
    VIDEO = "video"
    PDF = "pdf"
    FILE = "file"
    AUDIO = "audio"
    LINK_PREVIEW = "link_preview"
    UNSUPPORTED = "unsupported"


class ApiColor(str, Enum):
    """
    Notionのカラー設定

    テキストカラーと背景カラーの両方を含む
    """

    DEFAULT = "default"
    GRAY = "gray"
    BROWN = "brown"
    ORANGE = "orange"
    YELLOW = "yellow"
    GREEN = "green"
    BLUE = "blue"
    PURPLE = "purple"
    PINK = "pink"
    RED = "red"
    DEFAULT_BACKGROUND = "default_background"
    GRAY_BACKGROUND = "gray_background"
    BROWN_BACKGROUND = "brown_background"
    ORANGE_BACKGROUND = "orange_background"
    YELLOW_BACKGROUND = "yellow_background"
    GREEN_BACKGROUND = "green_background"
    BLUE_BACKGROUND = "blue_background"
    PURPLE_BACKGROUND = "purple_background"
    PINK_BACKGROUND = "pink_background"
    RED_BACKGROUND = "red_background"


class BaseBlockObject(BaseModel):
    """
    すべてのブロックタイプに共通のフィールド

    各ブロックタイプは、このクラスを継承し、
    `type`フィールドをLiteralでオーバーライドします。
    """

    object: Literal["block"] = Field("block", description="オブジェクトタイプ")
    id: StrictStr = Field(..., description="ブロックID")
    type: BlockType = Field(..., description="ブロックタイプ")
    created_time: StrictStr = Field(..., description="作成日時（ISO 8601形式）")
    created_by: PartialUser = Field(..., description="作成者")
    last_edited_time: StrictStr = Field(..., description="最終編集日時（ISO 8601形式）")
    last_edited_by: PartialUser = Field(..., description="最終編集者")
    parent: NotionParent = Field(..., description="親オブジェクト")
    has_children: StrictBool = Field(False, description="子ブロックの有無")
    archived: StrictBool = Field(False, description="アーカイブフラグ")
    in_trash: StrictBool = Field(False, description="ゴミ箱フラグ")


class PartialBlock(BaseModel):
    """部分的なブロック情報"""

    object: Literal["block"] = Field("block", description="オブジェクトタイプ")
    id: StrictStr = Field(..., description="ブロックID")



"""
サポートされていないブロックの定義
"""

from __future__ import annotations

from typing import Literal

from pydantic import BaseModel, Field

from .base import BaseBlockObject, BlockType


class EmptyObject(BaseModel):
    """空のオブジェクト"""

    pass


class UnsupportedBlock(BaseBlockObject):
    """サポートされていないブロック"""

    type: Literal[BlockType.UNSUPPORTED] = Field(
        BlockType.UNSUPPORTED, description="ブロックタイプ"
    )
    unsupported: EmptyObject = Field(
        default_factory=EmptyObject, description="空のコンテンツ"
    )



"""
プロパティ型の定義

PageObjectResponseで使用されるプロパティ値の型定義。
"""

from typing import Union

from ..properties import RelationProperty, RollupProperty
from ..properties.base_properties import (
    ButtonProperty,
    CheckboxProperty,
    CreatedByProperty,
    CreatedTimeProperty,
    DateProperty,
    EmailProperty,
    FormulaProperty,
    FilesProperty,
    LastEditedByProperty,
    LastEditedTimeProperty,
    LastVisitedTimeProperty,
    MultiSelectProperty,
    LocationProperty,
    NumberProperty,
    PeopleProperty,
    PhoneNumberProperty,
    PlaceProperty,
    RichTextProperty,
    SelectProperty,
    StatusProperty,
    TitleProperty,
    UniqueIdProperty,
    UrlProperty,
    VerificationProperty,
)


# PageObjectResponse の properties で使用されるプロパティ値の型
PropertyType = Union[
    ButtonProperty,
    CheckboxProperty,
    CreatedByProperty,
    CreatedTimeProperty,
    DateProperty,
    EmailProperty,
    FormulaProperty,
    FilesProperty,
    LastEditedByProperty,
    LastEditedTimeProperty,
    LastVisitedTimeProperty,
    MultiSelectProperty,
    LocationProperty,
    NumberProperty,
    PeopleProperty,
    PhoneNumberProperty,
    PlaceProperty,
    RichTextProperty,
    SelectProperty,
    StatusProperty,
    TitleProperty,
    UniqueIdProperty,
    UrlProperty,
    VerificationProperty,
    RelationProperty,
    RollupProperty,
]



from typing import Literal

from pydantic import BaseModel, Field

from .date_info import DateInfo
from .user import PartialUser


class VerificationUnverified(BaseModel):
    """未検証状態のverification"""

    state: Literal["unverified"] = Field("unverified", description="検証状態")
    date: None = Field(None, description="検証日")
    verified_by: None = Field(None, description="検証者")


class VerificationInfo(BaseModel):
    """検証済み/期限切れのverification情報"""

    state: Literal["verified", "expired"] = Field(
        ..., description="検証状態"
    )
    date: DateInfo | None = Field(None, description="検証日")
    verified_by: PartialUser | None = Field(None, description="検証者")


Verification = VerificationUnverified | VerificationInfo
"""Notionのverification値"""



"""Unit tests for FileUploadObject response type."""

import pytest
from notion_py_client.responses.file_upload import (
    FileUploadObject,
    FileUploadCreatedBy,
    FileImportResult,
    FileImportError,
)
from notion_py_client.responses.list_response import ListFileUploadsResponse


class TestFileUploadCreatedBy:
    """Test FileUploadCreatedBy."""

    def test_parse_created_by_person(self):
        """Test parsing created_by with person type."""
        data = {
            "id": "user_123",
            "type": "person",
        }

        result = FileUploadCreatedBy.model_validate(data)

        assert result.id == "user_123"
        assert result.type == "person"

    def test_parse_created_by_bot(self):
        """Test parsing created_by with bot type."""
        data = {
            "id": "bot_456",
            "type": "bot",
        }

        result = FileUploadCreatedBy.model_validate(data)

        assert result.type == "bot"

    def test_parse_created_by_agent(self):
        """Test parsing created_by with agent type."""
        data = {
            "id": "agent_789",
            "type": "agent",
        }

        result = FileUploadCreatedBy.model_validate(data)

        assert result.type == "agent"


class TestFileImportError:
    """Test FileImportError."""

    def test_parse_file_import_error(self):
        """Test parsing a file import error."""
        data = {
            "type": "validation_error",
            "code": "invalid_file_type",
            "message": "The file type is not supported",
            "parameter": "file",
            "status_code": 400,
        }

        result = FileImportError.model_validate(data)

        assert result.type == "validation_error"
        assert result.code == "invalid_file_type"
        assert result.message == "The file type is not supported"
        assert result.parameter == "file"
        assert result.status_code == 400

    def test_parse_file_import_error_minimal(self):
        """Test parsing a file import error with minimal fields."""
        data = {
            "type": "internal_system_error",
            "code": "internal_error",
            "message": "An internal error occurred",
        }

        result = FileImportError.model_validate(data)

        assert result.type == "internal_system_error"
        assert result.parameter is None
        assert result.status_code is None


class TestFileImportResult:
    """Test FileImportResult."""

    def test_parse_file_import_result_success(self):
        """Test parsing a successful file import result."""
        data = {
            "imported_time": "2025-01-01T12:00:00.000Z",
            "type": "success",
            "success": {"pages_imported": 5},
        }

        result = FileImportResult.model_validate(data)

        assert result.type == "success"
        assert result.imported_time == "2025-01-01T12:00:00.000Z"
        assert result.success == {"pages_imported": 5}
        assert result.error is None

    def test_parse_file_import_result_error(self):
        """Test parsing a failed file import result."""
        data = {
            "imported_time": "2025-01-01T12:00:00.000Z",
            "type": "error",
            "error": {
                "type": "upload_error",
                "code": "upload_failed",
                "message": "Failed to upload file",
            },
        }

        result = FileImportResult.model_validate(data)

        assert result.type == "error"
        assert result.error.type == "upload_error"
        assert result.success is None


class TestFileUploadObject:
    """Test FileUploadObject."""

    def test_parse_file_upload_pending(self):
        """Test parsing a file upload object with pending status."""
        data = {
            "object": "file_upload",
            "id": "file_123",
            "created_time": "2025-01-01T00:00:00.000Z",
            "created_by": {
                "id": "user_123",
                "type": "person",
            },
            "last_edited_time": "2025-01-01T00:00:00.000Z",
            "archived": False,
            "expiry_time": "2025-01-02T00:00:00.000Z",
            "status": "pending",
            "filename": "document.pdf",
            "content_type": "application/pdf",
            "content_length": 1024,
            "upload_url": "https://example.com/upload",
        }

        result = FileUploadObject.model_validate(data)

        assert result.object == "file_upload"
        assert result.id == "file_123"
        assert result.status == "pending"
        assert result.filename == "document.pdf"
        assert result.content_type == "application/pdf"
        assert result.content_length == 1024
        assert result.upload_url == "https://example.com/upload"
        assert result.archived is False

    def test_parse_file_upload_uploaded(self):
        """Test parsing a file upload object with uploaded status."""
        data = {
            "object": "file_upload",
            "id": "file_456",
            "created_time": "2025-01-01T00:00:00.000Z",
            "created_by": {
                "id": "bot_789",
                "type": "bot",
            },
            "last_edited_time": "2025-01-01T00:10:00.000Z",
            "archived": False,
            "expiry_time": None,
            "status": "uploaded",
            "filename": "image.png",
            "content_type": "image/png",
            "content_length": 2048,
            "complete_url": "https://example.com/complete",
        }

        result = FileUploadObject.model_validate(data)

        assert result.status == "uploaded"
        assert result.complete_url == "https://example.com/complete"
        assert result.expiry_time is None

    def test_parse_file_upload_with_import_result(self):
        """Test parsing a file upload with import result."""
        data = {
            "object": "file_upload",
            "id": "file_789",
            "created_time": "2025-01-01T00:00:00.000Z",
            "created_by": {
                "id": "user_456",
                "type": "person",
            },
            "last_edited_time": "2025-01-01T00:15:00.000Z",
            "archived": False,
            "expiry_time": None,
            "status": "uploaded",
            "file_import_result": {
                "imported_time": "2025-01-01T00:15:00.000Z",
                "type": "success",
                "success": {"pages_imported": 3},
            },
        }

        result = FileUploadObject.model_validate(data)

        assert result.file_import_result is not None
        assert result.file_import_result.type == "success"
        assert result.file_import_result.success["pages_imported"] == 3

    def test_parse_file_upload_failed(self):
        """Test parsing a failed file upload."""
        data = {
            "object": "file_upload",
            "id": "file_failed",
            "created_time": "2025-01-01T00:00:00.000Z",
            "created_by": {
                "id": "user_123",
                "type": "person",
            },
            "last_edited_time": "2025-01-01T00:05:00.000Z",
            "archived": False,
            "expiry_time": None,
            "status": "failed",
        }

        result = FileUploadObject.model_validate(data)

        assert result.status == "failed"
        assert result.filename is None

    def test_parse_file_upload_with_parts(self):
        """Test parsing a file upload with multi-part info."""
        data = {
            "object": "file_upload",
            "id": "file_multipart",
            "created_time": "2025-01-01T00:00:00.000Z",
            "created_by": {
                "id": "user_123",
                "type": "person",
            },
            "last_edited_time": "2025-01-01T00:00:00.000Z",
            "archived": False,
            "expiry_time": "2025-01-02T00:00:00.000Z",
            "status": "pending",
            "number_of_parts": {"total": 10, "sent": 5},
        }

        result = FileUploadObject.model_validate(data)

        assert result.number_of_parts is not None
        assert result.number_of_parts["total"] == 10
        assert result.number_of_parts["sent"] == 5


class TestListFileUploadsResponse:
    """Test ListFileUploadsResponse."""

    def test_parse_list_file_uploads_response(self):
        """Test parsing a list of file uploads."""
        data = {
            "object": "list",
            "results": [
                {
                    "object": "file_upload",
                    "id": "file_1",
                    "created_time": "2025-01-01T00:00:00.000Z",
                    "created_by": {"id": "user_123", "type": "person"},
                    "last_edited_time": "2025-01-01T00:00:00.000Z",
                    "archived": False,
                    "expiry_time": None,
                    "status": "uploaded",
                    "filename": "file1.pdf",
                    "content_type": "application/pdf",
                    "content_length": 1024,
                },
                {
                    "object": "file_upload",
                    "id": "file_2",
                    "created_time": "2025-01-01T00:05:00.000Z",
                    "created_by": {"id": "user_456", "type": "person"},
                    "last_edited_time": "2025-01-01T00:05:00.000Z",
                    "archived": False,
                    "expiry_time": "2025-01-02T00:00:00.000Z",
                    "status": "pending",
                },
            ],
            "next_cursor": "cursor_123",
            "has_more": True,
            "type": "file_upload",
        }

        result = ListFileUploadsResponse.model_validate(data)

        assert result.object == "list"
        assert result.has_more is True
        assert result.next_cursor == "cursor_123"
        assert result.type == "file_upload"
        assert len(result.results) == 2
        assert isinstance(result.results[0], FileUploadObject)
        assert result.results[0].status == "uploaded"
        assert result.results[1].status == "pending"

    def test_parse_empty_list_file_uploads(self):
        """Test parsing an empty file uploads list."""
        data = {
            "object": "list",
            "results": [],
            "next_cursor": None,
            "has_more": False,
            "type": "file_upload",
        }

        result = ListFileUploadsResponse.model_validate(data)

        assert len(result.results) == 0
        assert result.has_more is False
        assert result.next_cursor is None



"""Unit tests for NotionMapper and Field."""

import pytest
from datetime import date
from typing import cast
from pydantic import BaseModel, Field as PydanticField, StrictStr
from notion_py_client.helper import NotionMapper, NotionPropertyDescriptor, Field
from notion_py_client.requests.property_requests import (
    TitlePropertyRequest,
    StatusPropertyRequest,
    NumberPropertyRequest,
)
from notion_py_client.requests.common import SelectPropertyItemRequest
from notion_py_client.responses.property_types import (
    TitleProperty,
    StatusProperty,
    NumberProperty,
)
from notion_py_client.properties.base_properties._base_property import (
    NotionPropertyType,
)
from notion_py_client.requests.page_requests import (
    CreatePageParameters,
    UpdatePageParameters,
)
from notion_py_client.responses.page import NotionPage
from notion_py_client.models import RichTextItem, PartialUser, StatusOption
from notion_py_client.models.primitives import Text, Annotations


class Task(BaseModel):
    """Test domain model."""

    id: StrictStr
    name: StrictStr
    status: StrictStr
    priority: int


class TaskMapper(NotionMapper[Task]):
    """Test mapper implementation."""

    name_field: NotionPropertyDescriptor[TitleProperty, TitlePropertyRequest, str] = (
        Field(
            notion_name="Name",
            parser=lambda p: p.title[0].plain_text if p.title else "",
            request_builder=lambda v: TitlePropertyRequest(
                title=[{"type": "text", "text": {"content": v}}]
            ),
        )
    )

    status_field: NotionPropertyDescriptor[
        StatusProperty, StatusPropertyRequest, str
    ] = Field(
        notion_name="Status",
        parser=lambda p: p.status.name if p.status else "",
        request_builder=lambda v: StatusPropertyRequest(
            status=SelectPropertyItemRequest(name=v)
        ),
    )

    priority_field: NotionPropertyDescriptor[
        NumberProperty, NumberPropertyRequest, int
    ] = Field(
        notion_name="Priority",
        parser=lambda p: int(p.number) if p.number else 0,
        request_builder=lambda v: NumberPropertyRequest(number=v),
    )

    def to_domain(self, notion_page: NotionPage) -> Task:
        """Convert Notion page to domain model."""
        props = notion_page.properties
        return Task(
            id=notion_page.id,
            name=self.name_field.parse(cast(TitleProperty, props["Name"])),
            status=self.status_field.parse(cast(StatusProperty, props["Status"])),
            priority=self.priority_field.parse(cast(NumberProperty, props["Priority"])),
        )

    def build_update_properties(self, model: Task) -> UpdatePageParameters:
        """Build update parameters from domain model."""
        return UpdatePageParameters(
            page_id=model.id,
            properties={
                self.name_field.notion_name: self.name_field.build_request(model.name),
                self.status_field.notion_name: self.status_field.build_request(
                    model.status
                ),
                self.priority_field.notion_name: self.priority_field.build_request(
                    model.priority
                ),
            },
        )

    def build_create_properties(
        self, datasource_id: str, model: Task
    ) -> CreatePageParameters:
        """Build create parameters from domain model."""
        return CreatePageParameters(
            parent={"type": "database_id", "database_id": datasource_id},
            properties={
                self.name_field.notion_name: self.name_field.build_request(model.name),
                self.status_field.notion_name: self.status_field.build_request(
                    model.status
                ),
                self.priority_field.notion_name: self.priority_field.build_request(
                    model.priority
                ),
            },
        )


class TestField:
    """Test Field factory function."""

    def test_field_creation_parser_only(self):
        """Test Field creates NotionPropertyDescriptor with parser only."""
        field = Field(
            notion_name="Test",
            parser=lambda p: "test",
        )

        assert isinstance(field, NotionPropertyDescriptor)
        assert field.notion_name == "Test"
        assert field.parser is not None
        assert field.request_builder is not None  # Default lambda

    def test_field_creation_with_both(self):
        """Test Field with both parser and request_builder."""
        field = Field(
            notion_name="Title",
            parser=lambda p: "test",
            request_builder=lambda v: TitlePropertyRequest(
                title=[{"type": "text", "text": {"content": v}}]
            ),
        )

        assert field.notion_name == "Title"
        assert field.parser is not None
        assert field.request_builder is not None

    def test_field_request_builder_works(self):
        """Test that request_builder produces correct output."""
        field = Field(
            notion_name="Status",
            parser=lambda p: getattr(p, "status", None) and getattr(p.status, "name", ""),  # type: ignore
            request_builder=lambda v: StatusPropertyRequest(
                status=SelectPropertyItemRequest(name=v)
            ),
        )

        request = field.build_request("In Progress")
        assert isinstance(request, StatusPropertyRequest)
        result = request.model_dump(by_alias=True, exclude_none=True)
        assert result["status"]["name"] == "In Progress"

    def test_field_parser_readonly(self):
        """Test read-only field (parser only, no request_builder)."""
        field = Field(
            notion_name="Formula",
            parser=lambda p: getattr(getattr(p, "formula", None), "number", 0),  # type: ignore
        )

        assert field.notion_name == "Formula"
        # Default request_builder should exist but return None
        result = field.build_request(100)
        assert result is None

    def test_field_writeonly(self):
        """Test write-only field (request_builder only, no parser)."""
        field = Field(
            notion_name="WriteOnly",
            request_builder=lambda v: NumberPropertyRequest(number=v),  # type: ignore
        )

        assert field.notion_name == "WriteOnly"
        # Default parser should exist but return None
        # (Cannot test parse without actual property object)


class TestNotionPropertyDescriptor:
    """Test NotionPropertyDescriptor class."""

    def test_descriptor_initialization(self):
        """Test descriptor can be initialized."""
        descriptor = NotionPropertyDescriptor(
            notion_name="Test",
            parser=lambda p: "value",
            request_builder=lambda v: TitlePropertyRequest(
                title=[{"type": "text", "text": {"content": v}}]
            ),
        )

        assert descriptor.notion_name == "Test"

    def test_descriptor_parse_method(self):
        """Test parse method works."""
        descriptor = NotionPropertyDescriptor(
            notion_name="Title",
            parser=lambda p: p.title[0].plain_text if p.title else "",  # type: ignore
            request_builder=lambda v: TitlePropertyRequest(
                title=[{"type": "text", "text": {"content": v}}]
            ),
        )

        # Create mock TitleProperty
        mock_property = TitleProperty(
            id="test-id",
            type=NotionPropertyType.TITLE,
            title=[
                RichTextItem(
                    type="text",
                    text=Text(content="Test Title", link=None),
                    plain_text="Test Title",
                    href=None,
                    mention=None,
                    equation=None,
                    annotations=Annotations(
                        bold=False,
                        italic=False,
                        strikethrough=False,
                        underline=False,
                        code=False,
                        color="default",
                    ),
                )
            ],
        )

        result = descriptor.parse(mock_property)
        assert result == "Test Title"

    def test_descriptor_build_request_method(self):
        """Test build_request method works."""
        descriptor = NotionPropertyDescriptor(
            notion_name="Priority",
            parser=lambda p: int(p.number) if p.number else 0,  # type: ignore
            request_builder=lambda v: NumberPropertyRequest(number=v),  # type: ignore
        )

        request = descriptor.build_request(5)
        assert isinstance(request, NumberPropertyRequest)
        assert request.number == 5


class TestNotionMapper:
    """Test NotionMapper abstract base class."""

    def test_mapper_initialization(self):
        """Test mapper can be initialized."""
        mapper = TaskMapper()

        assert mapper.name_field.notion_name == "Name"
        assert mapper.status_field.notion_name == "Status"
        assert mapper.priority_field.notion_name == "Priority"

    def test_field_descriptors_accessible(self):
        """Test field descriptors are accessible as class attributes."""
        mapper = TaskMapper()

        # Fields should be accessible
        assert hasattr(mapper, "name_field")
        assert hasattr(mapper, "status_field")
        assert hasattr(mapper, "priority_field")

        # Check notion_name values
        assert mapper.name_field.notion_name == "Name"
        assert mapper.status_field.notion_name == "Status"
        assert mapper.priority_field.notion_name == "Priority"

    def test_mapper_with_mock_page(self):
        """Test to_domain conversion with mock NotionPage."""
        from notion_py_client.models import StatusOption

        # Create mock properties
        mock_properties = {
            "Name": TitleProperty(
                id="title-id",
                type=NotionPropertyType.TITLE,
                title=[
                    RichTextItem(
                        type="text",
                        text=Text(content="Test Task", link=None),
                        plain_text="Test Task",
                        href=None,
                        mention=None,
                        equation=None,
                        annotations=Annotations(
                            bold=False,
                            italic=False,
                            strikethrough=False,
                            underline=False,
                            code=False,
                            color="default",
                        ),
                    )
                ],
            ),
            "Status": StatusProperty(
                id="status-id",
                type=NotionPropertyType.STATUS,
                status=StatusOption(
                    id="status-opt-id",
                    name="In Progress",
                    color="blue",
                    description=None,
                ),
            ),
            "Priority": NumberProperty(
                id="number-id", type=NotionPropertyType.NUMBER, number=3
            ),
        }

        # Create mock NotionPage
        mock_page = NotionPage(
            object="page",
            id="test-page-id",
            created_time="2024-01-01T00:00:00.000Z",
            created_by=PartialUser(object="user", id="user-123"),
            last_edited_time="2024-01-01T00:00:00.000Z",
            last_edited_by=PartialUser(object="user", id="user-123"),
            archived=False,
            in_trash=False,
            is_locked=False,
            icon=None,
            cover=None,
            properties=mock_properties,
            parent={"type": "database_id", "database_id": "test-db-id"},
            url="https://notion.so/test",
            public_url=None,
            request_id=None,
        )

        # Test conversion
        mapper = TaskMapper()
        task = mapper.to_domain(mock_page)

        assert isinstance(task, Task)
        assert task.id == "test-page-id"
        assert task.name == "Test Task"
        assert task.status == "In Progress"
        assert task.priority == 3

    def test_mapper_build_update_properties(self):
        """Test build_update_properties method."""
        mapper = TaskMapper()
        task = Task(id="page-123", name="Updated Task", status="Done", priority=5)

        params = mapper.build_update_properties(task)

        assert isinstance(params, UpdatePageParameters)
        assert params.page_id == "page-123"
        assert params.properties is not None
        assert "Name" in params.properties
        assert "Status" in params.properties
        assert "Priority" in params.properties

        # Verify request types
        assert isinstance(params.properties["Name"], TitlePropertyRequest)
        assert isinstance(params.properties["Status"], StatusPropertyRequest)
        assert isinstance(params.properties["Priority"], NumberPropertyRequest)

    def test_mapper_build_create_properties(self):
        """Test build_create_properties method."""
        mapper = TaskMapper()
        task = Task(id="", name="New Task", status="Todo", priority=1)

        params = mapper.build_create_properties("db-123", task)

        assert isinstance(params, CreatePageParameters)
        # Type narrow to DatabaseParent
        parent = params.parent
        assert parent["type"] == "database_id"
        if parent["type"] == "database_id":  # type: ignore
            assert parent["database_id"] == "db-123"

        assert params.properties is not None
        assert "Name" in params.properties
        assert "Status" in params.properties
        assert "Priority" in params.properties


class TestMapperEdgeCases:
    """Test edge cases and error handling."""

    def test_empty_title_property(self):
        """Test parsing empty title property."""
        mapper = TaskMapper()
        empty_title = TitleProperty(
            id="title-id", type=NotionPropertyType.TITLE, title=[]
        )

        result = mapper.name_field.parse(empty_title)
        assert result == ""

    def test_none_status_property(self):
        """Test parsing status property with None value."""
        mapper = TaskMapper()
        none_status = StatusProperty(
            id="status-id", type=NotionPropertyType.STATUS, status=None
        )

        result = mapper.status_field.parse(none_status)
        assert result == ""

    def test_none_number_property(self):
        """Test parsing number property with None value."""
        mapper = TaskMapper()
        none_number = NumberProperty(
            id="number-id", type=NotionPropertyType.NUMBER, number=None
        )

        result = mapper.priority_field.parse(none_number)
        assert result == 0



from .annotation import Annotations
from .custom_emoji import CustomEmoji
from .empty_object import EmptyObject
from .equation import Equation
from .link_mention import LinkMention
from .link_preview import LinkPreviewMention
from .mention import Mention
from .template_mention import TemplateMention, TemplateMentionDate, TemplateMentionUser
from .text_item import Link, Text

__all__ = [
    "Annotations",
    "CustomEmoji",
    "EmptyObject",
    "Equation",
    "LinkMention",
    "LinkPreviewMention",
    "Mention",
    "TemplateMention",
    "TemplateMentionDate",
    "TemplateMentionUser",
    "Text",
    "Link",
]



"""Property request types for creating/updating Notion pages.

Page作成・更新時に使用するプロパティ値のリクエスト型定義。
TypeScript SDKの `UpdatePageBodyParameters.properties` に対応。

Note:
    以下のプロパティ型は読み取り専用のため、Request型は存在しません:
    - Formula (計算式による自動生成)
    - Rollup (他プロパティの集計)
    - CreatedBy, CreatedTime (作成情報)
    - LastEditedBy, LastEditedTime (更新情報)
    - UniqueId (自動生成ID)
    - Button (アクション用)
    - Verification (検証用)
"""

from typing import Any, Literal, Union

from pydantic import BaseModel

from .common import (
    DateRequest,
    GroupObjectRequest,
    IdRequest,
    InternalOrExternalFileWithNameRequest,
    FileUploadWithOptionalNameRequest,
    PartialUserObjectRequest,
    RelationItemRequest,
    SelectPropertyItemRequest,
    StringRequest,
    TextRequest,
)


class TitlePropertyRequest(BaseModel):
    """Titleプロパティのリクエスト."""

    title: list[Any]  # RichTextItemRequest
    type: Literal["title"] | None = None


class RichTextPropertyRequest(BaseModel):
    """RichTextプロパティのリクエスト."""

    rich_text: list[Any]  # RichTextItemRequest
    type: Literal["rich_text"] | None = None


class NumberPropertyRequest(BaseModel):
    """Numberプロパティのリクエスト."""

    number: float | None
    type: Literal["number"] | None = None


class UrlPropertyRequest(BaseModel):
    """URLプロパティのリクエスト."""

    url: TextRequest | None
    type: Literal["url"] | None = None


class SelectPropertyRequest(BaseModel):
    """Selectプロパティのリクエスト.

    選択肢をIDまたは名前で指定可能。
    Noneの場合は選択解除。
    """

    select: SelectPropertyItemRequest | None
    type: Literal["select"] | None = None


class MultiSelectPropertyRequest(BaseModel):
    """MultiSelectプロパティのリクエスト.

    複数の選択肢をIDまたは名前で指定可能。
    """

    multi_select: list[SelectPropertyItemRequest]
    type: Literal["multi_select"] | None = None


class StatusPropertyRequest(BaseModel):
    """Statusプロパティのリクエスト.

    ステータスをIDまたは名前で指定可能。
    Noneの場合はステータス解除。
    """

    status: SelectPropertyItemRequest | None
    type: Literal["status"] | None = None


class PeoplePropertyRequest(BaseModel):
    """Peopleプロパティのリクエスト.

    ユーザーまたはグループを指定可能。
    """

    people: list[PartialUserObjectRequest | GroupObjectRequest]
    type: Literal["people"] | None = None


class EmailPropertyRequest(BaseModel):
    """Emailプロパティのリクエスト."""

    email: StringRequest | None
    type: Literal["email"] | None = None


class PhoneNumberPropertyRequest(BaseModel):
    """PhoneNumberプロパティのリクエスト."""

    phone_number: StringRequest | None
    type: Literal["phone_number"] | None = None


class DatePropertyRequest(BaseModel):
    """Dateプロパティのリクエスト."""

    date: DateRequest | None
    type: Literal["date"] | None = None


class CheckboxPropertyRequest(BaseModel):
    """Checkboxプロパティのリクエスト."""

    checkbox: bool
    type: Literal["checkbox"] | None = None


class RelationPropertyRequest(BaseModel):
    """Relationプロパティのリクエスト.

    関連するページのIDのリストを指定。
    """

    relation: list[RelationItemRequest]
    type: Literal["relation"] | None = None


class FilesPropertyRequest(BaseModel):
    """Filesプロパティのリクエスト."""

    files: list[
        InternalOrExternalFileWithNameRequest | FileUploadWithOptionalNameRequest
    ]
    type: Literal["files"] | None = None


# すべてのプロパティリクエストのUnion型
PropertyRequest = Union[
    TitlePropertyRequest,
    RichTextPropertyRequest,
    NumberPropertyRequest,
    UrlPropertyRequest,
    SelectPropertyRequest,
    MultiSelectPropertyRequest,
    StatusPropertyRequest,
    PeoplePropertyRequest,
    EmailPropertyRequest,
    PhoneNumberPropertyRequest,
    DatePropertyRequest,
    CheckboxPropertyRequest,
    RelationPropertyRequest,
    FilesPropertyRequest,
]



from typing import Literal

from pydantic import Field

from ._base_config import BasePropertyConfig
from .select_config import SelectOptions
from ....properties.base_properties._base_property import NotionPropertyType


class MultiSelectOptions(SelectOptions):
    """multi_selectオプション定義"""


class MultiSelectPropertyConfig(
    BasePropertyConfig[Literal[NotionPropertyType.MULTI_SELECT]]
):
    """Notionのmulti_selectプロパティ設定"""

    type: Literal[NotionPropertyType.MULTI_SELECT] = Field(
        NotionPropertyType.MULTI_SELECT, description="プロパティタイプ"
    )
    multi_select: MultiSelectOptions = Field(
        default_factory=MultiSelectOptions, description="multi_select設定"
    )



from typing import Literal
from pydantic import Field

from ...models import SelectOption
from ._base_property import BaseProperty, NotionPropertyType


class SelectProperty(BaseProperty[Literal[NotionPropertyType.SELECT]]):
    """Notionのselectプロパティ"""

    type: Literal[NotionPropertyType.SELECT] = Field(
        NotionPropertyType.SELECT, description="プロパティタイプ"
    )

    select: SelectOption | None = Field(
        None, description="選択されたオプション（設定されていない場合はnull）"
    )

    def get_display_value(self) -> str | int | float | bool | None:
        """選択されたオプション名を取得

        Returns:
            str | None: 選択されたオプション名。未設定の場合はNone
        """
        if self.select is None:
            return None
        return self.select.name



from pydantic import BaseModel, Field, StrictStr


class DateInfo(BaseModel):
    """Notionの日付情報"""

    start: StrictStr = Field(..., description="開始日（ISO8601形式）")
    end: StrictStr | None = Field(
        None, description="終了日（ISO8601形式、範囲でない場合はnull）"
    )
    time_zone: StrictStr | None = Field(None, description="タイムゾーン")



"""
メディア系ブロックの定義

Embed, Bookmark, Image, Video, PDF, File, Audio, LinkPreview
"""

from __future__ import annotations

from enum import Enum
from typing import Literal

from pydantic import BaseModel, Field, StrictStr

from .base import BaseBlockObject, BlockType
from ..models.rich_text_item import RichTextItem
from ..models.file import InternalFile


class MediaFileType(str, Enum):
    """メディアファイルタイプ"""

    EXTERNAL = "external"
    FILE = "file"


# ============================================
# Content Models
# ============================================


class MediaContentWithUrlAndCaption(BaseModel):
    """URLとキャプションを持つメディアコンテンツ"""

    url: StrictStr = Field(..., description="URL")
    caption: list[RichTextItem] = Field(..., description="キャプション")


class ExternalFileReference(BaseModel):
    """外部ファイル参照"""

    url: StrictStr = Field(..., description="URL")


class ExternalMediaContentWithFileAndCaption(BaseModel):
    """外部ファイルとキャプションを持つメディアコンテンツ"""

    type: Literal[MediaFileType.EXTERNAL] = Field(
        MediaFileType.EXTERNAL, description="ファイルタイプ"
    )
    external: ExternalFileReference = Field(..., description="外部ファイル")
    caption: list[RichTextItem] = Field(..., description="キャプション")


class FileMediaContentWithFileAndCaption(BaseModel):
    """内部ファイルとキャプションを持つメディアコンテンツ"""

    type: Literal[MediaFileType.FILE] = Field(
        MediaFileType.FILE, description="ファイルタイプ"
    )
    file: InternalFile = Field(..., description="内部ファイル")
    caption: list[RichTextItem] = Field(..., description="キャプション")


MediaContentWithFileAndCaption = (
    ExternalMediaContentWithFileAndCaption | FileMediaContentWithFileAndCaption
)


class ExternalMediaContentWithFileNameAndCaption(BaseModel):
    """外部ファイル（ファイル名付き）とキャプションを持つメディアコンテンツ"""

    type: Literal[MediaFileType.EXTERNAL] = Field(
        MediaFileType.EXTERNAL, description="ファイルタイプ"
    )
    external: ExternalFileReference = Field(..., description="外部ファイル")
    caption: list[RichTextItem] = Field(..., description="キャプション")
    name: StrictStr = Field(..., description="ファイル名")


class FileMediaContentWithFileNameAndCaption(BaseModel):
    """内部ファイル（ファイル名付き）とキャプションを持つメディアコンテンツ"""

    type: Literal[MediaFileType.FILE] = Field(
        MediaFileType.FILE, description="ファイルタイプ"
    )
    file: InternalFile = Field(..., description="内部ファイル")
    caption: list[RichTextItem] = Field(..., description="キャプション")
    name: StrictStr = Field(..., description="ファイル名")


MediaContentWithFileNameAndCaption = (
    ExternalMediaContentWithFileNameAndCaption | FileMediaContentWithFileNameAndCaption
)


class MediaContentWithUrl(BaseModel):
    """URLのみを持つメディアコンテンツ"""

    url: StrictStr = Field(..., description="URL")


# ============================================
# Media Blocks
# ============================================


class EmbedBlock(BaseBlockObject):
    """埋め込みブロック"""

    type: Literal[BlockType.EMBED] = Field(
        BlockType.EMBED, description="ブロックタイプ"
    )
    embed: MediaContentWithUrlAndCaption = Field(..., description="埋め込みコンテンツ")


class BookmarkBlock(BaseBlockObject):
    """ブックマークブロック"""

    type: Literal[BlockType.BOOKMARK] = Field(
        BlockType.BOOKMARK, description="ブロックタイプ"
    )
    bookmark: MediaContentWithUrlAndCaption = Field(
        ..., description="ブックマークコンテンツ"
    )


class ImageBlock(BaseBlockObject):
    """画像ブロック"""

    type: Literal[BlockType.IMAGE] = Field(
        BlockType.IMAGE, description="ブロックタイプ"
    )
    image: MediaContentWithFileAndCaption = Field(..., description="画像コンテンツ")


class VideoBlock(BaseBlockObject):
    """動画ブロック"""

    type: Literal[BlockType.VIDEO] = Field(
        BlockType.VIDEO, description="ブロックタイプ"
    )
    video: MediaContentWithFileAndCaption = Field(..., description="動画コンテンツ")


class PdfBlock(BaseBlockObject):
    """PDFブロック"""

    type: Literal[BlockType.PDF] = Field(BlockType.PDF, description="ブロックタイプ")
    pdf: MediaContentWithFileAndCaption = Field(..., description="PDFコンテンツ")


class FileBlock(BaseBlockObject):
    """ファイルブロック"""

    type: Literal[BlockType.FILE] = Field(BlockType.FILE, description="ブロックタイプ")
    file: MediaContentWithFileNameAndCaption = Field(
        ..., description="ファイルコンテンツ"
    )


class AudioBlock(BaseBlockObject):
    """音声ブロック"""

    type: Literal[BlockType.AUDIO] = Field(
        BlockType.AUDIO, description="ブロックタイプ"
    )
    audio: MediaContentWithFileAndCaption = Field(..., description="音声コンテンツ")


class LinkPreviewBlock(BaseBlockObject):
    """リンクプレビューブロック"""

    type: Literal[BlockType.LINK_PREVIEW] = Field(
        BlockType.LINK_PREVIEW, description="ブロックタイプ"
    )
    link_preview: MediaContentWithUrl = Field(
        ..., description="リンクプレビューコンテンツ"
    )



from typing import Literal

from pydantic import BaseModel, Field, StrictStr

from ._base_config import BasePropertyConfig
from ....properties.base_properties._base_property import NotionPropertyType


class NumberOptions(BaseModel):
    """numberプロパティの追加設定"""

    format: StrictStr | None = Field(None, description="数値フォーマット")


class NumberPropertyConfig(BasePropertyConfig[Literal[NotionPropertyType.NUMBER]]):
    """Notionのnumberプロパティ設定"""

    type: Literal[NotionPropertyType.NUMBER] = Field(
        NotionPropertyType.NUMBER, description="プロパティタイプ"
    )
    number: NumberOptions = Field(
        default_factory=lambda: NumberOptions(format=None),
        description="数値プロパティ設定",
    )



from typing import Union

from .base_configs import (
    BasePropertyConfig,
    ButtonPropertyConfig,
    CheckboxPropertyConfig,
    CreatedByPropertyConfig,
    CreatedTimePropertyConfig,
    DatePropertyConfig,
    EmailPropertyConfig,
    FilesPropertyConfig,
    FormulaPropertyConfig,
    LastEditedByPropertyConfig,
    LastEditedTimePropertyConfig,
    MultiSelectOptions,
    MultiSelectPropertyConfig,
    NumberPropertyConfig,
    PeoplePropertyConfig,
    PhoneNumberPropertyConfig,
    RichTextPropertyConfig,
    SelectOptionConfig,
    SelectOptions,
    SelectPropertyConfig,
    StatusPropertyConfig,
    TitlePropertyConfig,
    UniqueIdPropertyConfig,
    UrlPropertyConfig,
    VerificationPropertyConfig,
    LocationPropertyConfig,
    LastVisitedTimePropertyConfig,
    PlacePropertyConfig,
)
from .relation_config import (
    RelationConfig,
    RelationDualConfig,
    RelationPropertyConfig,
    RelationSingleConfig,
)
from .rollup_config import RollupPropertyConfig, RollupSettings

PropertyConfig = Union[
    ButtonPropertyConfig,
    CheckboxPropertyConfig,
    CreatedByPropertyConfig,
    CreatedTimePropertyConfig,
    DatePropertyConfig,
    EmailPropertyConfig,
    FilesPropertyConfig,
    FormulaPropertyConfig,
    LastEditedByPropertyConfig,
    LastEditedTimePropertyConfig,
    MultiSelectPropertyConfig,
    NumberPropertyConfig,
    PeoplePropertyConfig,
    PhoneNumberPropertyConfig,
    RelationPropertyConfig,
    RichTextPropertyConfig,
    RollupPropertyConfig,
    SelectPropertyConfig,
    StatusPropertyConfig,
    TitlePropertyConfig,
    UniqueIdPropertyConfig,
    UrlPropertyConfig,
    VerificationPropertyConfig,
    LocationPropertyConfig,
    LastVisitedTimePropertyConfig,
    PlacePropertyConfig,
]

# データベースプロパティ設定のレスポンス型（PropertyConfigのエイリアス）
DatabasePropertyConfigResponse = PropertyConfig

__all__ = [
    "BasePropertyConfig",
    "ButtonPropertyConfig",
    "CheckboxPropertyConfig",
    "CreatedByPropertyConfig",
    "CreatedTimePropertyConfig",
    "DatePropertyConfig",
    "EmailPropertyConfig",
    "FilesPropertyConfig",
    "FormulaPropertyConfig",
    "LastEditedByPropertyConfig",
    "LastEditedTimePropertyConfig",
    "SelectOptionConfig",
    "SelectOptions",
    "MultiSelectOptions",
    "MultiSelectPropertyConfig",
    "NumberPropertyConfig",
    "PeoplePropertyConfig",
    "PhoneNumberPropertyConfig",
    "RelationConfig",
    "RelationSingleConfig",
    "RelationDualConfig",
    "RelationPropertyConfig",
    "RichTextPropertyConfig",
    "RollupPropertyConfig",
    "RollupSettings",
    "SelectPropertyConfig",
    "StatusPropertyConfig",
    "TitlePropertyConfig",
    "UniqueIdPropertyConfig",
    "UrlPropertyConfig",
    "VerificationPropertyConfig",
    "LocationPropertyConfig",
    "LastVisitedTimePropertyConfig",
    "PlacePropertyConfig",
    "PropertyConfig",
    "DatabasePropertyConfigResponse",
]



from typing import Literal

from pydantic import BaseModel, Field, StrictStr

from .base_configs import BasePropertyConfig
from ...models.primitives import EmptyObject
from ...properties.base_properties._base_property import NotionPropertyType


class RelationSingleConfig(BaseModel):
    """単方向のrelation設定"""

    type: Literal["single_property"] = Field(
        "single_property", description="Relation設定タイプ"
    )
    data_source_id: StrictStr = Field(..., description="関連先データソースID")
    single_property: EmptyObject = Field(
        default_factory=EmptyObject, description="単方向設定"
    )


class RelationDualDetails(BaseModel):
    """双方向relationの同期設定"""

    synced_property_id: StrictStr | None = Field(None, description="同期先プロパティID")
    synced_property_name: StrictStr | None = Field(
        None, description="同期先プロパティ名"
    )


class RelationDualConfig(BaseModel):
    """双方向relation設定"""

    type: Literal["dual_property"] = Field(
        "dual_property", description="Relation設定タイプ"
    )
    data_source_id: StrictStr = Field(..., description="関連先データソースID")
    dual_property: RelationDualDetails = Field(
        default_factory=lambda: RelationDualDetails(
            synced_property_id=None, synced_property_name=None
        ),
        description="双方向設定",
    )


RelationConfig = RelationSingleConfig | RelationDualConfig


class RelationPropertyConfig(BasePropertyConfig[Literal[NotionPropertyType.RELATION]]):
    """Notionのrelationプロパティ設定"""

    type: Literal[NotionPropertyType.RELATION] = Field(
        NotionPropertyType.RELATION, description="プロパティタイプ"
    )
    relation: RelationConfig



# notion_async_client.py
from __future__ import annotations
import asyncio
import base64
import json
from collections.abc import Sequence
from enum import Enum, auto
from typing import (
    Any,
    Callable,
    Iterable,
    Literal,
    Mapping,
    TypedDict,
    Union,
    cast,
    NotRequired,
)

from pydantic import TypeAdapter

import httpx

from .api_types import (
    CreateDatabaseParameters,
    DatabaseQuerySort,
    QueryDatabaseParameters,
    RetrieveDatabaseParameters,
    RetrievePageParameters,
    SearchParameters,
    UpdateDatabaseParameters,
)
from .filters import FilterCondition
from .requests.page_requests import CreatePageParameters, UpdatePageParameters
from .responses.database import NotionDatabase
from .responses.datasource import DataSource
from .responses.page import NotionPage
from .responses.file_upload import FileUploadObject
from .responses.property_item import PropertyItemListResponse, PropertyItemObject
from .responses.list_response import (
    QueryDatabaseResponse,
    QueryDataSourceResponse,
    ListUsersResponse,
    SearchResponse,
    ListFileUploadsResponse,
)
from .models.user import PartialUser, User


# ========== logging ==========
class LogLevel(Enum):
    DEBUG = auto()
    INFO = auto()
    WARN = auto()
    ERROR = auto()


def _severity(lv: LogLevel) -> int:
    return {
        LogLevel.DEBUG: 10,
        LogLevel.INFO: 20,
        LogLevel.WARN: 30,
        LogLevel.ERROR: 40,
    }[lv]


Logger = Callable[[LogLevel, str, Mapping[str, Any]], None]


def make_console_logger(prefix: str = "notionhq-client") -> Logger:
    def _log(level: LogLevel, message: str, extra: Mapping[str, Any]) -> None:
        print(f"[{prefix}] {level.name:5s} {message} :: {dict(extra)}")

    return _log


# ========== errors ==========
class APIErrorCode(str, Enum):
    Unauthorized = "unauthorized"
    RestrictedResource = "restricted_resource"
    ObjectNotFound = "object_not_found"
    RateLimited = "rate_limited"
    InvalidJSON = "invalid_json"
    InvalidRequestURL = "invalid_request_url"
    InvalidRequest = "invalid_request"
    ValidationError = "validation_error"
    ConflictError = "conflict_error"
    InternalServerError = "internal_server_error"
    ServiceUnavailable = "service_unavailable"


class ClientErrorCode(str, Enum):
    RequestTimeout = "notionhq_client_request_timeout"
    ResponseError = "notionhq_client_response_error"


NotionErrorCode = Union[APIErrorCode, ClientErrorCode]


class NotionClientErrorBase(Exception):
    code: NotionErrorCode


class RequestTimeoutError(NotionClientErrorBase):
    code = ClientErrorCode.RequestTimeout
    name = "RequestTimeoutError"

    def __init__(self, message: str = "Request to Notion API has timed out"):
        super().__init__(message)


class HTTPResponseError(NotionClientErrorBase):
    name = "HTTPResponseError"

    def __init__(
        self,
        *,
        code: NotionErrorCode,
        status: int,
        message: str,
        headers: Mapping[str, str],
        raw_body_text: str,
        additional_data: Mapping[str, Any] | None = None,
        request_id: str | None = None,
    ):
        super().__init__(message)
        self.code = code
        self.status = status
        self.headers = headers
        self.body = raw_body_text
        self.additional_data = additional_data
        self.request_id = request_id


class UnknownHTTPResponseError(HTTPResponseError):
    name = "UnknownHTTPResponseError"

    def __init__(
        self,
        *,
        status: int,
        headers: Mapping[str, str],
        raw_body_text: str,
        message: str | None = None,
    ):
        super().__init__(
            code=ClientErrorCode.ResponseError,
            status=status,
            message=message or f"Request to Notion API failed with status: {status}",
            headers=headers,
            raw_body_text=raw_body_text,
        )


class APIResponseError(HTTPResponseError):
    name = "APIResponseError"


def _parse_api_error_body(body_text: str) -> dict[str, Any] | None:
    try:
        data = json.loads(body_text)
    except Exception:
        return None
    if not isinstance(data, dict):
        return None
    code = data.get("code")
    message = data.get("message")
    if (
        isinstance(code, str)
        and code in set(c.value for c in APIErrorCode)
        and isinstance(message, str)
    ):
        return {
            "code": APIErrorCode(code),
            "message": message,
            "additional_data": data.get("additional_data"),
            "request_id": data.get("request_id"),
        }
    return None


def build_request_error(response: httpx.Response, body_text: str) -> HTTPResponseError:
    parsed = _parse_api_error_body(body_text)
    if parsed:
        return APIResponseError(
            code=parsed["code"],
            status=response.status_code,
            message=parsed["message"],
            headers=dict(response.headers),
            raw_body_text=body_text,
            additional_data=parsed.get("additional_data"),
            request_id=parsed.get("request_id"),
        )
    return UnknownHTTPResponseError(
        status=response.status_code,
        headers=dict(response.headers),
        raw_body_text=body_text,
    )


# ========== utils ==========
def pick(d: Mapping[str, Any], keys: Iterable[str]) -> dict[str, Any]:
    return {k: v for k, v in d.items() if k in keys and v is not None}


def _normalize_query(
    q: Mapping[str, Any] | None,
) -> Sequence[tuple[str, str]] | None:
    """
    クエリパラメータを正規化してhttpx互換の形式に変換。
    多値クエリパラメータ（同じキーで複数の値）を保持するためSequenceを返す。
    """
    if not q:
        return None
    out: list[tuple[str, str]] = []
    for k, v in q.items():
        if v is None:
            continue
        if isinstance(v, (list, tuple)):
            for item in v:
                out.append((k, str(item)))
        else:
            out.append((k, str(v)))
    return out


# ========== client ==========
# Method type (公式SDKと同じ)
Method = Literal["get", "post", "patch", "delete"]

# QueryParams type (公式SDKと同じ)
QueryParams = dict[str, str | int | list[str]]


class FileParam(TypedDict, total=False):
    """ファイルパラメータ (公式SDKのFileParam型)"""

    filename: str
    data: str | bytes  # Pythonでは bytes を使用


class ClientOptions(TypedDict, total=False):
    """
    Notionクライアントのオプション

    Examples:
        ```python
        # 最小構成
        client = NotionAsyncClient(auth="secret_xxx")

        # カスタム設定
        client = NotionAsyncClient(
            auth="secret_xxx",
            options={
                "timeout_ms": 30000,
                "log_level": LogLevel.DEBUG,
                "notion_version": "2025-09-03"
            }
        )
        ```
    """

    timeout_ms: NotRequired[int]
    base_url: NotRequired[str]
    log_level: NotRequired[LogLevel]
    logger: NotRequired[Logger | None]
    notion_version: NotRequired[str]
    user_agent: NotRequired[str]


# AuthParam type (公式SDKと同じ)
AuthParam = Union[
    str,  # Bearer token
    dict[
        str, str
    ],  # {"client_id": "...", "client_secret": "..."}  (OAuthのBasic認可用)
]


class NotionAsyncClient:
    """
    公式JSクライアントの構成を踏襲した async Python クライアント。

    Notion API 2025-09-03に対応:
    - databases: データベース（複数のデータソースを持つコンテナ）の管理
    - dataSources: データソース（実際のデータとスキーマを持つ）の管理・クエリ
    - pages: ページの作成・取得・更新
    - blocks: ブロックの操作
    - users: ユーザー情報の取得
    - comments: コメントの管理
    - fileUploads: ファイルアップロード
    - search: 検索

    重要な概念変更:
    - 旧API: Database = データとスキーマを持つ単一のエンティティ
    - 新API: Database = 複数のDataSourceを持つコンテナ
            DataSource = データとスキーマを持つエンティティ（旧Databaseに相当）

    使い方:
    1. databases.retrieve()でdata_sources一覧を取得
    2. dataSources.query()でデータを取得（旧databases.query()）
    3. dataSources.retrieve()でスキーマを取得（旧databases.retrieve()のproperties部分）
    """

    default_notion_version = "2025-09-03"

    def __init__(self, auth: str, options: ClientOptions | None = None):
        """
        Notionクライアントの初期化

        Args:
            auth: Notion APIの認証トークン（必須）
            options: その他のオプション設定

        Examples:
            ```python
            # シンプルな使い方
            client = NotionAsyncClient(auth="secret_xxx")

            # オプション付き
            client = NotionAsyncClient(
                auth="secret_xxx",
                options={
                    "timeout_ms": 30000,
                    "log_level": LogLevel.DEBUG
                }
            )
            ```
        """
        opts = options or {}
        self._auth = auth
        self._timeout_ms = opts.get("timeout_ms", 60_000)
        base_url = opts.get("base_url", "https://api.notion.com")
        self._prefix_url = f"{base_url.rstrip('/')}/v1/"
        self._notion_version = opts.get("notion_version") or self.default_notion_version
        self._logger = opts.get("logger") or make_console_logger("notionhq-client")
        self._log_level = opts.get("log_level", LogLevel.WARN)
        self._user_agent = opts.get("user_agent", "notionhq-client-python/0.1.0")

        self._client = httpx.AsyncClient(timeout=self._timeout_ms / 1000)

        # ------- public API groups（JSに合わせた名前/階層）-------
        self.blocks = _BlocksAPI(self)
        self.databases = _DatabasesAPI(self)
        self.dataSources = _DataSourcesAPI(self)
        self.pages = _PagesAPI(self)
        self.users = _UsersAPI(self)
        self.comments = _CommentsAPI(self)
        self.fileUploads = _FileUploadsAPI(self)

    # ---------- lifecycle ----------
    async def aclose(self) -> None:
        await self._client.aclose()

    async def __aenter__(self) -> "NotionAsyncClient":
        return self

    async def __aexit__(self, exc_type: Any, exc: Any, tb: Any) -> None:
        await self.aclose()

    # ---------- auth header ----------
    def _auth_as_headers(self, auth: AuthParam | None) -> dict[str, str]:
        headers: dict[str, str] = {}
        auth_value = auth if auth is not None else self._auth
        if isinstance(auth_value, str):
            headers["authorization"] = f"Bearer {auth_value}"
        elif isinstance(auth_value, dict):
            cid = auth_value.get("client_id", "")
            sec = auth_value.get("client_secret", "")
            token = base64.b64encode(f"{cid}:{sec}".encode()).decode()
            headers["authorization"] = f"Basic {token}"
        return headers

    def _log(self, level: LogLevel, message: str, extra: Mapping[str, Any]) -> None:
        if _severity(level) >= _severity(self._log_level):
            self._logger(level, message, extra)

    # ---------- request core ----------
    async def request(
        self,
        *,
        path: str,
        method: Method,
        query: QueryParams | None = None,
        body: dict[str, Any] | None = None,
        form_data_params: dict[str, str | FileParam] | None = None,
        headers: dict[str, str] | None = None,
        auth: AuthParam | None = None,
    ) -> dict[str, Any]:
        url = f"{self._prefix_url}{path}"
        self._log(LogLevel.INFO, "request start", {"method": method, "path": path})

        # headers (優先順: std headers < auth headers < request headers)
        # 右側が優先されるため、標準→認証→呼び出し側の順で展開
        req_headers: dict[str, str] = {
            "Notion-Version": self._notion_version,
            "user-agent": self._user_agent,
            **self._auth_as_headers(auth),
            **(headers or {}),
        }

        json_payload: str | None = None
        files_payload: dict[str, tuple[str | None, Any, str | None]] | None = None

        if form_data_params:
            # multipart/form-data
            # httpx は files に (name, (filename, content, content_type)) を渡す
            files_payload = {}
            for k, v in form_data_params.items():
                if isinstance(v, str):
                    files_payload[k] = (None, v, None)
                else:
                    filename = v.get("filename") if isinstance(v, dict) else None
                    data_value = v.get("data") if isinstance(v, dict) else None
                    if isinstance(data_value, str):
                        data_value = data_value.encode()
                    files_payload[k] = (filename, data_value, None)
        else:
            if body and len(body) > 0:
                json_payload = json.dumps(body)
                req_headers["content-type"] = "application/json"

        # クエリ整形（多値クエリを保持するためリストのまま渡す）
        params = _normalize_query(query)

        # タイムアウト（JSの RequestTimeoutError 相当）
        async def _send() -> httpx.Response:
            return await self._client.request(
                method.upper(),
                url,
                headers=req_headers,
                content=json_payload,
                params=cast(
                    Any, params
                ),  # httpxはSequence[tuple[str, str]]を受け入れる
                files=files_payload,
            )

        try:
            response: httpx.Response = await asyncio.wait_for(
                _send(), timeout=self._timeout_ms / 1000
            )
        except asyncio.TimeoutError:
            self._log(
                LogLevel.WARN,
                "request fail",
                {"code": ClientErrorCode.RequestTimeout, "message": "timeout"},
            )
            raise RequestTimeoutError()

        text = await response.aread()
        text_str = text.decode(errors="replace")

        if not (200 <= response.status_code < 300):
            err = build_request_error(response, text_str)
            self._log(
                LogLevel.WARN,
                "request fail",
                {
                    "code": getattr(err, "code", "unknown"),
                    "message": str(err),
                    **(
                        {"requestId": getattr(err, "request_id")}
                        if getattr(err, "request_id", None)
                        else {}
                    ),
                },
            )
            raise err

        data: dict[str, Any] = json.loads(text_str) if text_str else {}
        self._log(
            LogLevel.INFO,
            "request success",
            {
                "method": method,
                "path": path,
                **(
                    {"requestId": data.get("request_id")}
                    if data.get("request_id")
                    else {}
                ),
            },
        )
        return data

    # ---------- search ----------
    async def search(
        self, params: SearchParameters, *, auth: AuthParam | None = None
    ) -> SearchResponse:
        """
        Search pages and databases

        Args:
            params: 検索パラメータ（クエリ、フィルター、ソート）
            auth: 認証トークン（オプション）

        Examples:
            ```python
            from api_types import SearchParameters

            results = await client.search({
                "query": "プロジェクト",
                "filter": {"property": "object", "value": "page"},
                "page_size": 20
            })
            ```
        """
        response = await self.request(
            path="search", method="post", body=dict(params), auth=auth
        )
        # dict を SearchResponse に変換
        from .responses.page import PartialPage
        from .responses.database import PartialDatabase

        results = []
        for item in response.get("results", []):
            obj_type = item.get("object")
            if obj_type == "page":
                if "properties" in item:
                    results.append(NotionPage(**item))
                else:
                    results.append(PartialPage(**item))
            elif obj_type == "database":
                if "properties" in item:
                    results.append(NotionDatabase(**item))
                else:
                    results.append(PartialDatabase(**item))

        return SearchResponse(
            object="list",
            results=results,
            next_cursor=response.get("next_cursor"),
            has_more=response.get("has_more", False),
            type="page_or_database",
        )

    # ---------- oauth ----------
    async def oauth_token(
        self,
        *,
        grant_type: str,
        client_id: str,
        client_secret: str,
        code: str | None = None,
        redirect_uri: str | None = None,
        external_account: dict[str, str] | None = None,
    ) -> dict[str, Any]:
        """OAuth token endpoint"""
        body = pick(
            {
                "grant_type": grant_type,
                "code": code,
                "redirect_uri": redirect_uri,
                "external_account": external_account,
            },
            ["grant_type", "code", "redirect_uri", "external_account"],
        )
        return await self.request(
            path="oauth/token",
            method="post",
            body=body,
            auth={"client_id": client_id, "client_secret": client_secret},
        )

    async def oauth_introspect(
        self, *, token: str, client_id: str, client_secret: str
    ) -> dict[str, Any]:
        """OAuth introspect endpoint"""
        body = {"token": token}
        return await self.request(
            path="oauth/introspect",
            method="post",
            body=body,
            auth={"client_id": client_id, "client_secret": client_secret},
        )

    async def oauth_revoke(
        self, *, token: str, client_id: str, client_secret: str
    ) -> dict[str, Any]:
        """OAuth revoke endpoint"""
        body = {"token": token}
        return await self.request(
            path="oauth/revoke",
            method="post",
            body=body,
            auth={"client_id": client_id, "client_secret": client_secret},
        )


# ========== Endpoint groups（JSの形に合わせた薄い Facade） ==========
class _BlocksAPI:
    def __init__(self, client: NotionAsyncClient):
        self._c = client
        self.children = _BlockChildrenAPI(client)

    async def retrieve(
        self, *, block_id: str, auth: AuthParam | None = None
    ) -> dict[str, Any]:
        """Retrieve a block

        Returns:
            BlockObjectResponse | PartialBlockObjectResponse: ブロックオブジェクト
        """
        return await self._c.request(path=f"blocks/{block_id}", method="get", auth=auth)

    async def update(self, *, block_id: str, **body) -> dict[str, Any]:
        """Update a block

        Returns:
            BlockObjectResponse | PartialBlockObjectResponse: 更新されたブロックオブジェクト
        """
        return await self._c.request(
            path=f"blocks/{block_id}", method="patch", body=body
        )

    async def delete(
        self, *, block_id: str, auth: AuthParam | None = None
    ) -> dict[str, Any]:
        """Delete a block

        Returns:
            BlockObjectResponse | PartialBlockObjectResponse: 削除されたブロックオブジェクト（archived=True）
        """
        return await self._c.request(
            path=f"blocks/{block_id}", method="delete", auth=auth
        )


class _BlockChildrenAPI:
    def __init__(self, client: NotionAsyncClient):
        self._c = client

    async def append(
        self,
        *,
        block_id: str,
        children: list[dict[str, Any]],
        after: str | None = None,
        auth: AuthParam | None = None,
    ) -> dict[str, Any]:
        """Append block children

        Returns:
            AppendBlockChildrenResponse: 追加された子ブロックを含むレスポンス
        """
        body = {"children": children, **({"after": after} if after else {})}
        return await self._c.request(
            path=f"blocks/{block_id}/children", method="patch", body=body, auth=auth
        )

    async def list(
        self,
        *,
        block_id: str,
        start_cursor: str | None = None,
        page_size: int | None = None,
        auth: AuthParam | None = None,
    ) -> dict[str, Any]:
        """List block children

        Returns:
            ListBlockChildrenResponse: 子ブロックのリスト
        """
        query = pick(
            {"start_cursor": start_cursor, "page_size": page_size},
            ["start_cursor", "page_size"],
        )
        return await self._c.request(
            path=f"blocks/{block_id}/children", method="get", query=query, auth=auth
        )


class _DatabasesAPI:
    """
    Databases API - 2025-09-03対応

    データベースは複数のデータソースを持つコンテナです。
    - databases.retrieve(): データベースのメタ情報とdata_sources一覧を取得
    - databases.create(): 新しいデータベースと初期データソースを作成
    - databases.update(): データベースレベルの属性（title, icon, cover等）を更新

    データのクエリやスキーマ操作はdataSources APIを使用してください。
    """

    def __init__(self, client: NotionAsyncClient):
        self._c = client

    async def retrieve(
        self, params: RetrieveDatabaseParameters, *, auth: AuthParam | None = None
    ) -> NotionDatabase:
        """
        Retrieve a database (API version 2025-09-03)

        データベースのメタ情報とdata_sources一覧を取得します。
        propertiesは含まれません。スキーマ情報が必要な場合は
        dataSources.retrieve(data_source_id)を使用してください。

        Args:
            params: データベース取得パラメータ
            auth: 認証トークン（オプション）

        Returns:
            NotionDatabase: データベースオブジェクト（data_sources配列を含む）

        Examples:
            ```python
            # データベース情報を取得
            db = await client.databases.retrieve(
                {"database_id": "abc123"}
            )

            # データソース一覧を確認
            for ds in db.data_sources:
                print(f"Data Source: {ds['name']} (ID: {ds['id']})")

            # 最初のデータソースのスキーマを取得
            if db.data_sources:
                ds_id = db.data_sources[0]["id"]
                data_source = await client.dataSources.retrieve(
                    data_source_id=ds_id
                )
            ```
        """
        response = await self._c.request(
            path=f"databases/{params['database_id']}", method="get", auth=auth
        )
        return NotionDatabase(**response)

    async def create(
        self, params: CreateDatabaseParameters, *, auth: AuthParam | None = None
    ) -> NotionDatabase:
        """
        Create a database (API version 2025-09-03)

        新しいデータベースと初期データソースを作成します。
        initial_data_sourceにpropertiesを指定してください。

        Returns:
            NotionDatabase: 作成されたデータベースオブジェクト

        Examples:
            ```python
            db = await client.databases.create({
                "parent": {"type": "page_id", "page_id": "parent_page_id"},
                "title": [{"text": {"content": "My Database"}}],
                "initial_data_source": {
                    "properties": {
                        "Name": {"title": {}},
                        "Status": {"select": {"options": [...]}}
                    }
                }
            })
            ```
        """
        response = await self._c.request(
            path="databases", method="post", body=dict(params), auth=auth
        )
        return NotionDatabase(**response)

    async def query(
        self, params: QueryDatabaseParameters, *, auth: AuthParam | None = None
    ) -> QueryDatabaseResponse:
        """Legacy databases.query endpoint.

        If the client's Notion-Version is the latest (>= 2025-09-03), this
        method is considered legacy and will warn then raise, guiding callers
        to use dataSources.query instead. If an older Notion-Version is set,
        the request is forwarded to the legacy endpoint.
        """
        nv = self._c._notion_version
        latest = self._c.default_notion_version
        if nv >= latest:
            self._c._log(
                LogLevel.WARN,
                "databases.query is legacy; use dataSources.query instead",
                {"notion_version": nv, "required": f"< {latest}"},
            )
            raise RuntimeError(
                "databases.query is not available for Notion-Version >= 2025-09-03."
                " Use dataSources.query or set an older Notion-Version explicitly."
            )

        # Forward to legacy endpoint for older versions
        database_id = params["database_id"]
        body = pick(
            params,
            [
                "filter",
                "sorts",
                "start_cursor",
                "page_size",
                "filter_properties",
                "archived",
            ],
        )
        response = await self._c.request(
            path=f"databases/{database_id}/query", method="post", body=body, auth=auth
        )
        # Build QueryDatabaseResponse
        results: list[NotionPage] = [
            NotionPage(**item) for item in response.get("results", [])
        ]
        return QueryDatabaseResponse(
            object="list",
            results=results,
            next_cursor=response.get("next_cursor"),
            has_more=response.get("has_more", False),
            type="page_or_database",
        )

    async def update(
        self, params: UpdateDatabaseParameters, *, auth: AuthParam | None = None
    ) -> NotionDatabase:
        """
        Update a database (API version 2025-09-03)

        データベースレベルの属性を更新します。
        更新可能: title, icon, cover, is_inline, parent

        データソースのpropertiesを変更する場合は
        dataSources.update()を使用してください。

        Returns:
            NotionDatabase: 更新されたデータベースオブジェクト

        Examples:
            ```python
            # データベースのタイトルを変更
            db = await client.databases.update({
                "database_id": "abc123",
                "title": [{"text": {"content": "New Title"}}]
            })
            ```
        """
        database_id = params["database_id"]
        body = {k: v for k, v in params.items() if k != "database_id"}
        response = await self._c.request(
            path=f"databases/{database_id}",
            method="patch",
            body=body,
            auth=auth,
        )
        return NotionDatabase(**response)


class _DataSourcesAPI:
    """
    Data Sources API - 2025-09-03対応

    データソースは旧APIでの「データベース」の概念に相当します。
    - 実際のデータ（ページ）を持つ
    - スキーマ（properties）を持つ
    - クエリ、作成、更新が可能

    データベースは複数のデータソースを持つコンテナとなりました。
    """

    def __init__(self, client: NotionAsyncClient):
        self._c = client

    async def retrieve(
        self, *, data_source_id: str, auth: AuthParam | None = None
    ) -> DataSource:
        """
        Retrieve a data source (API version 2025-09-03)

        データソースのスキーマ情報（properties）を取得します。
        これは旧APIのdatabases.retrieve()に相当します。

        Args:
            data_source_id: データソースID
            auth: 認証トークン（オプション）

        Returns:
            DataSource: データソースオブジェクト（propertiesを含む）

        Examples:
            ```python
            # データソースのスキーマを取得
            ds = await client.dataSources.retrieve(
                data_source_id="ds_abc123"
            )

            # プロパティを確認
            for name, config in ds.properties.items():
                print(f"{name}: {config.type}")
            ```
        """
        response = await self._c.request(
            path=f"data_sources/{data_source_id}", method="get", auth=auth
        )
        return DataSource(**response)

    async def query(
        self,
        *,
        data_source_id: str,
        filter: FilterCondition | None = None,
        sorts: list[DatabaseQuerySort] | None = None,
        start_cursor: str | None = None,
        page_size: int | None = None,
        auth: AuthParam | None = None,
    ) -> QueryDataSourceResponse:
        """
        Query a data source (API version 2025-09-03)

        データソース内のページ（データ行）を取得します。
        これは旧APIのdatabases.query()に相当します。

        **型安全なフィルター作成:**
        公式TypeScript SDKの型定義に準拠したフィルター型を使用可能です。
        ```typescript
        // PropertyFilter (23種類)
        type PropertyFilter =
          | { title: TextPropertyFilter; property: string; type?: "title" }
          | { rich_text: TextPropertyFilter; property: string; type?: "rich_text" }
          | { number: NumberPropertyFilter; property: string; type?: "number" }
          // ... その他のプロパティタイプ

        // TimestampFilter
        type TimestampFilter =
          | { created_time: DatePropertyFilter; timestamp: "created_time" }
          | { last_edited_time: DatePropertyFilter; timestamp: "last_edited_time" }

        // CompoundFilter
        filter?: PropertyFilter | TimestampFilter
          | { or: Array<PropertyFilter | TimestampFilter> }
          | { and: Array<PropertyFilter | TimestampFilter> }
        ```

        Args:
            data_source_id: データソースID
            filter: フィルター条件（PropertyFilter | TimestampFilter | 複合フィルター）
                型安全な作成には `create_and_filter()`, `create_or_filter()` を使用
            sorts: ソート条件のリスト
                - `[{"property": "名前", "direction": "ascending"}]`
                - `[{"timestamp": "created_time", "direction": "descending"}]`
            start_cursor: ページネーション用カーソル
            page_size: 取得件数（最大100）
            auth: 認証トークン（オプション）

        Returns:
            QueryDataSourceResponse: クエリ結果（results, has_more, next_cursorを含む）

        Examples:
            ```python
            from notion_py_client import NotionAsyncClient
            from notion_py_client.filters import create_and_filter

            client = NotionAsyncClient(auth="secret_xxx")

            # シンプルなフィルター (直接辞書 - IDE補完あり)
            response = await client.dataSources.query(
                data_source_id="ds_abc123",
                filter={"property": "Status", "status": {"equals": "Done"}}
            )

            # 複数条件 (型安全なヘルパー関数 - IDE補完あり)
            response = await client.dataSources.query(
                data_source_id="ds_abc123",
                filter=create_and_filter(
                    {"property": "Status", "status": {"equals": "Active"}},
                    {"property": "Amount", "number": {"greater_than": 10000}},
                )
            )

            # タイムスタンプフィルター
            response = await client.dataSources.query(
                data_source_id="ds_abc123",
                filter={
                    "timestamp": "created_time",
                    "created_time": {"past_week": {}}
                }
            )

            # ページネーション
            for page in response.results:
                if isinstance(page, NotionPage):
                    print(f"Page: {page.id}")

            if response.has_more:
                next_response = await client.dataSources.query(
                    data_source_id="ds_abc123",
                    start_cursor=response.next_cursor
                )
            ```

        Note:
            IDE補完を活用するには、型安全な`create_and_filter()`などのヘルパー関数を使用してください。
            公式ドキュメント: https://developers.notion.com/reference/post-database-query
        """
        body = pick(
            {
                "filter": filter,
                "sorts": sorts,
                "start_cursor": start_cursor,
                "page_size": page_size,
            },
            ["filter", "sorts", "start_cursor", "page_size"],
        )
        response = await self._c.request(
            path=f"data_sources/{data_source_id}/query",
            method="post",
            body=body,
            auth=auth,
        )

        # dictの結果をNotionPage/PartialPageに変換
        from .responses.page import PartialPage

        results = []
        for item in response.get("results", []):
            if "properties" in item:
                results.append(NotionPage(**item))
            else:
                results.append(PartialPage(**item))

        return QueryDataSourceResponse(
            object="list",
            results=results,
            next_cursor=response.get("next_cursor"),
            has_more=response.get("has_more", False),
            type="page_or_data_source",
        )

    async def query_legacy(
        self,
        params: QueryDatabaseParameters,
        *,
        auth: AuthParam | None = None,
    ) -> QueryDataSourceResponse:
        """Compatibility wrapper (not for public use)."""
        # Forward to dataSources.query since legacy databases.query is replaced
        database_id = params["database_id"]
        body = pick(
            params,
            [
                "filter",
                "sorts",
                "start_cursor",
                "page_size",
                "filter_properties",
                "archived",
            ],
        )
        response = await self._c.request(
            path=f"databases/{database_id}/query", method="post", body=body, auth=auth
        )
        from .responses.page import PartialPage

        results = []
        for item in response.get("results", []):
            if "properties" in item:
                results.append(NotionPage(**item))
            else:
                results.append(PartialPage(**item))
        return QueryDataSourceResponse(
            object="list",
            results=results,
            next_cursor=response.get("next_cursor"),
            has_more=response.get("has_more", False),
            type="page_or_data_source",
        )

    async def create(
        self,
        *,
        parent: dict[str, Any],
        title: list[dict[str, Any]] | None = None,
        properties: dict[str, Any],
        auth: AuthParam | None = None,
        **kwargs,
    ) -> DataSource:
        """
        Create a data source (API version 2025-09-03)

        既存のデータベースに新しいデータソースを追加します。

        Args:
            parent: 親データベース（{"type": "database_id", "database_id": "..."}）
            title: データソースのタイトル
            properties: スキーマ定義
            auth: 認証トークン（オプション）

        Returns:
            DataSource: 作成されたデータソースオブジェクト

        Examples:
            ```python
            ds = await client.dataSources.create(
                parent={"type": "database_id", "database_id": "db_abc123"},
                title=[{"text": {"content": "New Data Source"}}],
                properties={
                    "Name": {"title": {}},
                    "Status": {"select": {"options": [...]}}
                }
            )
            ```
        """
        body = {
            "parent": parent,
            "properties": properties,
            **pick({"title": title, **kwargs}, ["title"]),
        }
        response = await self._c.request(
            path="data_sources", method="post", body=body, auth=auth
        )
        return DataSource(**response)

    async def update(
        self,
        *,
        data_source_id: str,
        title: list[dict[str, Any]] | None = None,
        properties: dict[str, Any] | None = None,
        auth: AuthParam | None = None,
        **kwargs,
    ) -> DataSource:
        """
        Update a data source (API version 2025-09-03)

        データソースのスキーマ（properties）やタイトルを更新します。

        Args:
            data_source_id: データソースID
            title: 新しいタイトル
            properties: 更新するプロパティ（既存のプロパティを削除するにはnullを指定）
            auth: 認証トークン（オプション）

        Returns:
            DataSource: 更新されたデータソースオブジェクト

        Examples:
            ```python
            # プロパティを追加・削除
            ds = await client.dataSources.update(
                data_source_id="ds_abc123",
                properties={
                    "New Field": {"checkbox": {}},  # 追加
                    "Old Field": None  # 削除
                }
            )
            ```
        """
        body = pick(
            {"title": title, "properties": properties, **kwargs},
            ["title", "properties"],
        )
        response = await self._c.request(
            path=f"data_sources/{data_source_id}", method="patch", body=body, auth=auth
        )
        return DataSource(**response)


class _PagesAPI:
    def __init__(self, client: NotionAsyncClient):
        self._c = client
        self.properties = _PagePropertiesAPI(client)

    async def create(
        self, params: CreatePageParameters, *, auth: AuthParam | None = None
    ) -> NotionPage:
        """Create a page

        Returns:
            NotionPage: 作成されたページオブジェクト
        """
        body = params.model_dump(exclude_none=True, by_alias=True)
        response = await self._c.request(
            path="pages", method="post", body=body, auth=auth
        )
        return NotionPage(**response)

    async def retrieve(
        self, params: RetrievePageParameters, *, auth: AuthParam | None = None
    ) -> NotionPage:
        """Retrieve a page

        Returns:
            NotionPage: ページオブジェクト
        """
        from typing import cast

        page_id = params["page_id"]
        query_params = {k: v for k, v in params.items() if k != "page_id"}
        response = await self._c.request(
            path=f"pages/{page_id}",
            method="get",
            query=cast(QueryParams, query_params) if query_params else None,
            auth=auth,
        )
        return NotionPage(**response)

    async def update(
        self, params: UpdatePageParameters, *, auth: AuthParam | None = None
    ) -> NotionPage:
        """Update a page

        Returns:
            NotionPage: 更新されたページオブジェクト
        """
        payload = params.model_dump(exclude_none=True, by_alias=True)
        page_id = payload.pop("page_id")
        response = await self._c.request(
            path=f"pages/{page_id}", method="patch", body=payload, auth=auth
        )
        return NotionPage(**response)


class _PagePropertiesAPI:
    def __init__(self, client: NotionAsyncClient):
        self._c = client

    async def retrieve(
        self,
        *,
        page_id: str,
        property_id: str,
        start_cursor: str | None = None,
        page_size: int | None = None,
        auth: AuthParam | None = None,
    ) -> PropertyItemListResponse | PropertyItemObject:
        """Retrieve a page property item

        Returns:
            PropertyItemObjectResponse: プロパティアイテムオブジェクト（型に応じてtitle, rich_text, number, date, people等の値を含む）
        """
        query = pick(
            {"start_cursor": start_cursor, "page_size": page_size},
            ["start_cursor", "page_size"],
        )
        res = await self._c.request(
            path=f"pages/{page_id}/properties/{property_id}",
            method="get",
            query=query,
            auth=auth,
        )
        if isinstance(res, dict) and res.get("object") == "list":
            return PropertyItemListResponse.model_validate(res)
        # PropertyItemObject is a Union, use TypeAdapter
        adapter = TypeAdapter(PropertyItemObject)
        return adapter.validate_python(res)


class _UsersAPI:
    def __init__(self, client: NotionAsyncClient):
        self._c = client

    async def retrieve(
        self, *, user_id: str, auth: AuthParam | None = None
    ) -> PartialUser | User:
        """Retrieve a user

        Returns:
            UserObjectResponse: ユーザーオブジェクト
        """
        res = await self._c.request(path=f"users/{user_id}", method="get", auth=auth)
        # User is a Union (PersonUser | BotUser), use TypeAdapter
        try:
            adapter = TypeAdapter(User)
            return adapter.validate_python(res)
        except Exception:
            return PartialUser.model_validate(res)

    async def list(
        self,
        *,
        start_cursor: str | None = None,
        page_size: int | None = None,
        auth: AuthParam | None = None,
    ) -> ListUsersResponse:
        """List all users

        Returns:
            ListUsersResponse: ユーザーのリスト
        """
        query = pick(
            {"start_cursor": start_cursor, "page_size": page_size},
            ["start_cursor", "page_size"],
        )
        response = await self._c.request(
            path="users", method="get", query=query, auth=auth
        )
        from .models.user import PartialUser

        return ListUsersResponse(
            object="list",
            results=[PartialUser(**user) for user in response.get("results", [])],
            next_cursor=response.get("next_cursor"),
            has_more=response.get("has_more", False),
            type="user",
        )

    async def me(self, *, auth: AuthParam | None = None) -> dict[str, Any]:
        """Get details about bot

        Returns:
            UserObjectResponse: ボットユーザーの詳細
        """
        return await self._c.request(path="users/me", method="get", auth=auth)


class _CommentsAPI:
    def __init__(self, client: NotionAsyncClient):
        self._c = client

    async def create(
        self,
        *,
        parent: dict[str, Any],
        rich_text: list[dict[str, Any]],
        discussion_id: str | None = None,
        auth: AuthParam | None = None,
    ) -> dict[str, Any]:
        """Create a comment

        Returns:
            CommentObjectResponse: 作成されたコメントオブジェクト
        """
        body = {
            "parent": parent,
            "rich_text": rich_text,
            **pick({"discussion_id": discussion_id}, ["discussion_id"]),
        }
        return await self._c.request(
            path="comments", method="post", body=body, auth=auth
        )

    async def list(
        self,
        *,
        block_id: str | None = None,
        start_cursor: str | None = None,
        page_size: int | None = None,
        auth: AuthParam | None = None,
    ) -> dict[str, Any]:
        """List comments

        Returns:
            ListCommentsResponse: コメントのリスト（object="list", type="comment", results=CommentObjectResponse[]）
        """
        query = pick(
            {
                "block_id": block_id,
                "start_cursor": start_cursor,
                "page_size": page_size,
            },
            ["block_id", "start_cursor", "page_size"],
        )
        return await self._c.request(
            path="comments", method="get", query=query, auth=auth
        )

    async def retrieve(
        self, *, comment_id: str, auth: AuthParam | None = None
    ) -> dict[str, Any]:
        """Retrieve a comment

        Returns:
            CommentObjectResponse: コメントオブジェクト
        """
        return await self._c.request(
            path=f"comments/{comment_id}", method="get", auth=auth
        )


class _FileUploadsAPI:
    def __init__(self, client: NotionAsyncClient):
        self._c = client

    async def create(
        self,
        *,
        name: str,
        size: int,
        mime_type: str | None = None,
        auth: AuthParam | None = None,
    ) -> FileUploadObject:
        """Create a file upload

        Returns:
            FileUploadObjectResponse: 作成されたファイルアップロードオブジェクト（upload_urlを含む）
        """
        body = {
            "name": name,
            "size": size,
            **pick({"mime_type": mime_type}, ["mime_type"]),
        }
        res = await self._c.request(
            path="file_uploads", method="post", body=body, auth=auth
        )
        return FileUploadObject(**res)

    async def retrieve(
        self, *, file_upload_id: str, auth: AuthParam | None = None
    ) -> FileUploadObject:
        """Retrieve a file upload

        Returns:
            FileUploadObjectResponse: ファイルアップロードオブジェクト（status, upload_urlなどを含む）
        """
        res = await self._c.request(
            path=f"file_uploads/{file_upload_id}", method="get", auth=auth
        )
        return FileUploadObject(**res)

    async def list(
        self,
        *,
        start_cursor: str | None = None,
        page_size: int | None = None,
        auth: AuthParam | None = None,
    ) -> ListFileUploadsResponse:
        """List file uploads

        Returns:
            ListFileUploadsResponse: ファイルアップロードのリスト（object="list", type="file_upload", results=FileUploadObjectResponse[]）
        """
        query = pick(
            {"start_cursor": start_cursor, "page_size": page_size},
            ["start_cursor", "page_size"],
        )
        res = await self._c.request(
            path="file_uploads", method="get", query=query, auth=auth
        )
        return ListFileUploadsResponse(
            object="list",
            results=[FileUploadObject(**it) for it in res.get("results", [])],
            next_cursor=res.get("next_cursor"),
            has_more=res.get("has_more", False),
            type="file_upload",
        )

    async def send(
        self,
        *,
        file_upload_id: str,
        file: dict[str, Any],
        part_number: str | None = None,
        auth: AuthParam | None = None,
    ) -> FileUploadObject:
        """
        Send a file upload

        Requires a `file_upload_id`, obtained from the `id` of the Create File
        Upload API response.

        The `file` parameter contains the raw file contents or Blob/File object
        under `file.data`, and an optional `file.filename` string.

        Supply a stringified `part_number` parameter when using file uploads
        in multi-part mode.

        This endpoint sends HTTP multipart/form-data instead of JSON parameters.

        Returns:
            FileUploadPartObjectResponse: アップロードされたパートのレスポンス
        """
        form_data = {
            "file": file,
            **pick({"part_number": part_number}, ["part_number"]),
        }
        res = await self._c.request(
            path=f"file_uploads/{file_upload_id}",
            method="post",
            form_data_params=form_data,
            auth=auth,
        )
        return FileUploadObject(**res)

    async def complete(
        self, *, file_upload_id: str, auth: AuthParam | None = None
    ) -> FileUploadObject:
        """Complete a file upload

        Returns:
            FileUploadObjectResponse: 完了したファイルアップロードオブジェクト（status="complete"）
        """
        res = await self._c.request(
            path=f"file_uploads/{file_upload_id}/complete",
            method="post",
            auth=auth,
        )
        return FileUploadObject(**res)



from typing import Literal
from pydantic import Field

from ...models import StatusOption
from ._base_property import BaseProperty, NotionPropertyType


class StatusProperty(BaseProperty[Literal[NotionPropertyType.STATUS]]):
    """Notionのstatusプロパティ"""

    type: Literal[NotionPropertyType.STATUS] = Field(
        NotionPropertyType.STATUS, description="プロパティタイプ"
    )

    status: StatusOption | None = Field(
        None, description="現在のステータス（設定されていない場合はnull）"
    )

    def get_display_value(self) -> str | int | float | bool | None:
        """選択されたステータス名を取得

        Returns:
            str | None: 選択されたステータス名。未設定の場合はNone
        """
        if self.status is None:
            return None
        return self.status.name



from typing import Optional

from pydantic import BaseModel, Field, StrictFloat, StrictInt, StrictStr


class LinkMention(BaseModel):
    """Notionのlink_mention詳細"""

    href: StrictStr = Field(..., description="リンクのhref")
    title: Optional[StrictStr] = Field(None, description="リンクタイトル")
    description: Optional[StrictStr] = Field(None, description="リンクの説明")
    link_author: Optional[StrictStr] = Field(None, description="リンクの作者")
    link_provider: Optional[StrictStr] = Field(None, description="リンクの提供元")
    thumbnail_url: Optional[StrictStr] = Field(None, description="サムネイルURL")
    icon_url: Optional[StrictStr] = Field(None, description="アイコンURL")
    iframe_url: Optional[StrictStr] = Field(None, description="iframeのURL")
    height: Optional[StrictFloat | StrictInt] = Field(None, description="プレビュー高さ")
    padding: Optional[StrictFloat | StrictInt] = Field(None, description="iframeパディング")
    padding_top: Optional[StrictFloat | StrictInt] = Field(
        None, description="iframeパディング上部"
    )



from abc import abstractmethod
from enum import Enum
from typing import Any, Generic, TypeVar

from pydantic import BaseModel, Field, StrictStr


class NotionPropertyType(str, Enum):
    """Notionプロパティタイプの列挙型"""

    TITLE = "title"
    RICH_TEXT = "rich_text"
    DATE = "date"
    NUMBER = "number"
    SELECT = "select"
    MULTI_SELECT = "multi_select"
    PEOPLE = "people"
    RELATION = "relation"
    URL = "url"
    CHECKBOX = "checkbox"
    FORMULA = "formula"
    STATUS = "status"
    ROLLUP = "rollup"
    BUTTON = "button"
    LAST_EDITED_TIME = "last_edited_time"
    EMAIL = "email"
    PHONE_NUMBER = "phone_number"
    FILES = "files"
    CREATED_BY = "created_by"
    CREATED_TIME = "created_time"
    LAST_EDITED_BY = "last_edited_by"
    UNIQUE_ID = "unique_id"
    VERIFICATION = "verification"
    # Newer property types
    LOCATION = "location"
    LAST_VISITED_TIME = "last_visited_time"
    PLACE = "place"


TPropertyType = TypeVar("TPropertyType", bound="NotionPropertyType")


class BaseProperty(BaseModel, Generic[TPropertyType]):
    """Notionプロパティのベースクラス"""

    id: StrictStr | None = Field(None, description="プロパティID")
    # 各サブクラスで特定の Literal[...] によって具体化される
    type: TPropertyType = Field(..., description="プロパティタイプ")

    @abstractmethod
    def get_display_value(self) -> str | int | float | bool | None:
        """Notion UI上でのプロパティの表示用の値を取得"""
        pass



from typing import Any, Literal
import json

from pydantic import Field

from ._base_property import BaseProperty, NotionPropertyType


class LocationProperty(BaseProperty[Literal[NotionPropertyType.LOCATION]]):
    """Notionのlocationプロパティ

    現在のNotion APIの更新に伴う新プロパティ。公式の返却形状は将来的に
    変更の可能性があるため、値は汎用の辞書として保持します。
    """

    type: Literal[NotionPropertyType.LOCATION] = Field(
        NotionPropertyType.LOCATION, description="プロパティタイプ"
    )

    location: dict[str, Any] | None = Field(
        default=None, description="ロケーション情報（API仕様未確定のため汎用辞書）"
    )

    def get_display_value(self) -> str | int | float | bool | None:
        """ロケーション情報を取得

        Returns:
            str | None: ロケーション情報の文字列表現。未設定の場合はNone
        """
        if self.location is None:
            return None
        try:
            return json.dumps(self.location, ensure_ascii=False, separators=(",", ":"))
        except Exception:
            return str(self.location)



from typing import Literal

from pydantic import Field, StrictStr

from ._base_property import BaseProperty, NotionPropertyType


class LastVisitedTimeProperty(
    BaseProperty[Literal[NotionPropertyType.LAST_VISITED_TIME]]
):
    """Notionのlast_visited_timeプロパティ"""

    type: Literal[NotionPropertyType.LAST_VISITED_TIME] = Field(
        NotionPropertyType.LAST_VISITED_TIME, description="プロパティタイプ"
    )

    last_visited_time: StrictStr | None = Field(
        None, description="最終表示日時（ISO 8601形式）"
    )

    def get_display_value(self) -> StrictStr | int | float | bool | None:
        """最終表示日時を取得

        Returns:
            StrictStr | None: 最終表示日時（未設定の場合はnull）
        """
        return self.last_visited_time



"""Integration tests for Domain Mapper.

These tests require actual Notion API data.
Set NOTION_API_TOKEN and TEST_DATASOURCE_ID environment variables to run these tests.

Usage:
    export NOTION_API_TOKEN="your_api_key"
    export TEST_DATASOURCE_ID="your_datasource_id"
    pytest test/integration/test_mapper.py -v
"""

import os
import pytest
from notion_py_client import NotionAsyncClient


pytestmark = pytest.mark.skipif(
    not os.getenv("NOTION_API_TOKEN"),
    reason="NOTION_API_TOKEN environment variable not set",
)


@pytest.fixture
def notion_client() -> NotionAsyncClient:
    """Create NotionAsyncClient with API key from environment."""

    api_key = os.getenv("NOTION_API_TOKEN")
    return NotionAsyncClient(auth=api_key)  # type: ignore


@pytest.fixture
def test_datasource_id():
    """Get test datasource ID from environment."""
    datasource_id = os.getenv("TEST_DATASOURCE_ID")
    if not datasource_id:
        pytest.skip("TEST_DATASOURCE_ID not set")
    return datasource_id


class TestMapperIntegration:
    """Integration tests for NotionMapper with real data."""

    @pytest.mark.asyncio
    async def test_query_and_parse_with_mapper(
        self, notion_client, test_datasource_id, mock_notion_mapper
    ):
        """Test querying datasource and parsing with mapper.

        Steps:
        1. Query pages from real datasource
        2. Use mapper to convert to domain models
        3. Verify domain model fields are correctly populated
        """
        # Query datasource
        response = await notion_client.dataSources.query(
            data_source_id=test_datasource_id, page_size=5
        )

        assert response is not None
        assert hasattr(response, "results")
        assert len(response.results) > 0

        # Parse first page with mapper
        first_page = response.results[0]

        # Convert to domain model
        try:
            domain_model = mock_notion_mapper.to_domain(first_page)

            # Verify domain model
            assert domain_model.id == first_page.id
            assert domain_model.assignee is None or isinstance(
                domain_model.assignee, str
            )
            assert domain_model.date_range is not None
            assert domain_model.project_range is not None

            print(f"Successfully parsed page: {domain_model.id}")
            print(f"  Assignee: {domain_model.assignee}")
            print(f"  Date Range: {domain_model.date_range}")
            print(f"  Project Range: {domain_model.project_range}")

        except ValueError as e:
            # Skip if page doesn't have required fields
            pytest.skip(f"Page missing required fields: {e}")

    @pytest.mark.asyncio
    async def test_query_multiple_pages_with_mapper(
        self, notion_client, test_datasource_id, mock_notion_mapper
    ):
        """Test parsing multiple pages from datasource.

        Verifies that mapper can handle various data patterns.
        """
        response = await notion_client.dataSources.query(
            data_source_id=test_datasource_id, page_size=10
        )

        assert response is not None
        assert len(response.results) > 0

        parsed_count = 0
        error_count = 0

        for page in response.results:
            try:
                domain_model = mock_notion_mapper.to_domain(page)
                parsed_count += 1

                # Basic validation
                assert domain_model.id == page.id
                assert isinstance(domain_model.project_range, list)

            except (ValueError, KeyError, AttributeError) as e:
                # Some pages may not have all required fields
                error_count += 1
                print(f"Failed to parse page {page.id}: {e}")

        print(f"Successfully parsed: {parsed_count}/{len(response.results)} pages")
        print(f"Parse errors: {error_count}")

        # At least some pages should be parseable
        assert parsed_count > 0



from typing import Any, Literal

from pydantic import Field

from ...models import FormulaResult
from ._base_property import BaseProperty, NotionPropertyType


class FormulaProperty(BaseProperty[Literal[NotionPropertyType.FORMULA]]):
    """Notionのformulaプロパティ"""

    type: Literal[NotionPropertyType.FORMULA] = Field(
        NotionPropertyType.FORMULA, description="プロパティタイプ"
    )

    formula: FormulaResult = Field(..., description="フォーミュラの計算結果")

    def get_display_value(self) -> str | int | float | bool | None:
        """フォーミュラの計算結果を取得

        Returns:
            str | int | float | bool | None: フォーミュラの計算結果
        
        Note:
            - フォーミュラの型に応じて、string, number, boolean, dateのいずれかの値を返す
            - date型の場合、startとendの両方が存在する場合は "start→end" の形式で返す
            - date型でstartのみ存在する場合はstartを返す
            - フォーミュラが未設定の場合はNoneを返す
        """
        if self.formula.type == "string":
            return self.formula.string
        elif self.formula.type == "number":
            return self.formula.number
        elif self.formula.type == "boolean":
            return self.formula.boolean
        elif self.formula.type == "date":
            if self.formula.date is None:
                return None
            if self.formula.date.start and self.formula.date.end:
                return f"{self.formula.date.start}→{self.formula.date.end}"
            else:
                return self.formula.date.start
        else:
            return None



from typing import Literal

from pydantic import Field

from ...models import UniqueId
from ._base_property import BaseProperty, NotionPropertyType


class UniqueIdProperty(BaseProperty[Literal[NotionPropertyType.UNIQUE_ID]]):
    """Notionのunique_idプロパティ"""

    type: Literal[NotionPropertyType.UNIQUE_ID] = Field(
        NotionPropertyType.UNIQUE_ID, description="プロパティタイプ"
    )

    unique_id: UniqueId = Field(..., description="ユニークIDの値")

    def get_display_value(self) -> str | int | float | bool | None:
        """ユニークIDの表示値を取得

        - prefix と number の両方がある場合は `PREFIX-<number>`
        - number のみ: `<number>`
        - prefix のみ: `prefix`
        - どちらも無い場合: None
        """
        prefix = getattr(self.unique_id, "prefix", None)
        number = getattr(self.unique_id, "number", None)
        if prefix and number is not None:
            return f"{prefix}-{number}"
        if number is not None:
            return str(number)
        if prefix:
            return prefix
        return None



"""
NotionページオブジェクトのPydanticモデル定義

TypeScript定義: PageObjectResponse

ページオブジェクトは、pages.retrieve APIなどで返される型です。
基本的にはDataSourceObjectResponseと同じ構造ですが、
より豊富な情報を含む可能性があります。
"""

from typing import Literal

from pydantic import BaseModel, Field, StrictBool, StrictStr

from ..models.user import PartialUser
from ..models.icon import NotionIcon
from ..models.cover import NotionCover
from ..models.parent import NotionParent
from .property_types import PropertyType


class NotionPage(BaseModel):
    """
    Notionページオブジェクト

    TypeScript: PageObjectResponse

    **重要**: NotionPage はデータベース内の実際のデータ行（レコード）、
    または通常のNotionページです。

    用途:
    - データベース内の1行のデータ
    - 通常のNotionページ
    - properties には実際のプロパティ値が入る

    取得方法:
    - pages.retrieve(page_id) で個別のページを取得
    - databases.query(database_id) でデータベース内のページ一覧を取得
    - pages.create() で新規ページを作成
    - pages.update() でページを更新

    Examples:
        ```python
        # ページを取得
        page = await client.pages.retrieve({"page_id": "abc123"})

        # プロパティにアクセス（実際の値が入っている）
        title_prop = page.properties.get("名前")
        if title_prop and hasattr(title_prop, "get_display_value"):
            title = title_prop.get_display_value()
            print(f"Title: {title}")

        # メタデータにアクセス
        print(f"Created: {page.created_time}")
        print(f"URL: {page.url}")
        ```
    """

    object: Literal["page"] = Field(..., description="オブジェクトタイプ")
    id: StrictStr = Field(..., description="ページID")
    created_time: StrictStr = Field(..., description="作成日時（ISO 8601形式）")
    last_edited_time: StrictStr = Field(..., description="最終編集日時（ISO 8601形式）")
    created_by: PartialUser = Field(..., description="作成者")
    last_edited_by: PartialUser = Field(..., description="最終編集者")
    parent: NotionParent = Field(..., description="親オブジェクト")
    archived: StrictBool = Field(False, description="アーカイブフラグ")
    in_trash: StrictBool = Field(False, description="ゴミ箱フラグ")
    is_locked: StrictBool = Field(False, description="ロックフラグ")
    properties: dict[str, PropertyType] = Field(..., description="プロパティ一覧")
    icon: NotionIcon | None = Field(None, description="アイコン")
    cover: NotionCover | None = Field(None, description="カバー画像")
    url: StrictStr = Field(..., description="ページURL")
    public_url: StrictStr | None = Field(None, description="パブリックURL")

    # ページ固有のプロパティ（オプション）
    request_id: StrictStr | None = Field(None, description="リクエストID")


class PartialPage(BaseModel):
    """部分的なページ情報"""

    object: Literal["page"] = Field(..., description="オブジェクトタイプ")
    id: StrictStr = Field(..., description="ページID")



from typing import Literal
from pydantic import Field

from ...models import RichTextItem
from ._base_property import BaseProperty, NotionPropertyType


class TitleProperty(BaseProperty[Literal[NotionPropertyType.TITLE]]):
    """Notionのtitleプロパティ"""

    type: Literal[NotionPropertyType.TITLE] = Field(
        NotionPropertyType.TITLE, description="プロパティタイプ"
    )

    title: list[RichTextItem] = Field(..., description="タイトルのRichText配列")

    def get_display_value(self) -> str | int | float | bool | None:
        """タイトルの内容を取得

        Returns:
            str | None: タイトルの内容を連結した文字列。タイトルが空の場合はNone
        """
        if len(self.title) == 0:
            return None
        return "".join(item.plain_text for item in self.title)



from .responses.datasource import DataSource, PartialDataSource
from .responses.database import NotionDatabase, PartialDatabase
from .responses.page import NotionPage, PartialPage
from .responses.file_upload import FileUploadObject
from .responses.property_item import PropertyItemObject, PropertyItemListResponse
from .blocks import (
    BlockObject,
    PartialBlock,
    BlockType,
    ApiColor,
    BaseBlockObject,
    # Text blocks
    ParagraphBlock,
    Heading1Block,
    Heading2Block,
    Heading3Block,
    BulletedListItemBlock,
    NumberedListItemBlock,
    QuoteBlock,
    ToDoBlock,
    ToggleBlock,
    TemplateBlock,
    # Special blocks
    SyncedBlockBlock,
    ChildPageBlock,
    ChildDatabaseBlock,
    EquationBlock,
    CodeBlock,
    CalloutBlock,
    # Layout blocks
    DividerBlock,
    BreadcrumbBlock,
    TableOfContentsBlock,
    ColumnListBlock,
    ColumnBlock,
    LinkToPageBlock,
    TableBlock,
    TableRowBlock,
    # Media blocks
    EmbedBlock,
    BookmarkBlock,
    ImageBlock,
    VideoBlock,
    PdfBlock,
    FileBlock,
    AudioBlock,
    LinkPreviewBlock,
    # Unsupported
    UnsupportedBlock,
)
from .filters import (
    # フィルター基底型
    PropertyFilter,
    TimestampFilter,
    FilterCondition,
    AndFilterDict,
    OrFilterDict,
    create_and_filter,
    create_or_filter,
    # プロパティフィルター
    PropertyFilterTitle,
    PropertyFilterRichText,
    PropertyFilterNumber,
    PropertyFilterCheckbox,
    PropertyFilterSelect,
    PropertyFilterMultiSelect,
    PropertyFilterStatus,
    PropertyFilterDate,
    PropertyFilterPeople,
    PropertyFilterFiles,
    PropertyFilterUrl,
    PropertyFilterEmail,
    PropertyFilterPhoneNumber,
    PropertyFilterRelation,
    PropertyFilterCreatedBy,
    PropertyFilterCreatedTime,
    PropertyFilterLastEditedBy,
    PropertyFilterLastEditedTime,
    PropertyFilterFormula,
    PropertyFilterUniqueId,
    PropertyFilterRollup,
    PropertyFilterVerification,
    # 個別フィルター条件
    TextPropertyFilter,
    NumberPropertyFilter,
    CheckboxPropertyFilter,
    SelectPropertyFilter,
    MultiSelectPropertyFilter,
    StatusPropertyFilter,
    DatePropertyFilter,
    PeoplePropertyFilter,
    RelationPropertyFilter,
    FormulaPropertyFilter,
    RollupPropertyFilter,
    ExistencePropertyFilter,
    # 具体的なフィルター
    TextFilterEquals,
    TextFilterContains,
    NumberFilterEquals,
    NumberFilterGreaterThan,
    DateFilterAfter,
    DateFilterBefore,
    RelationFilterContains,
)
from .requests import (
    NumberPropertyRequest,
    TitlePropertyRequest,
    RichTextPropertyRequest,
    CheckboxPropertyRequest,
    SelectPropertyRequest,
    MultiSelectPropertyRequest,
    StatusPropertyRequest,
    DatePropertyRequest,
    PeoplePropertyRequest,
    FilesPropertyRequest,
    UrlPropertyRequest,
    EmailPropertyRequest,
    PhoneNumberPropertyRequest,
    RelationPropertyRequest,
    UpdatePageParameters,
    CreatePageParameters,
    RelationItemRequest,
    DateRequest,
    IdRequest,
    SelectPropertyItemRequest,
    PartialUserObjectRequest,
    GroupObjectRequest,
    AnnotationRequest,
    PropertyRequest,
)
from .schema import (
    PeoplePropertyConfig,
    SelectPropertyConfig,
    MultiSelectPropertyConfig,
    MultiSelectOptions,
    StatusPropertyConfig,
    SelectOptionConfig,
    SelectOptions,
    DatePropertyConfig,
    NumberPropertyConfig,
    PhoneNumberPropertyConfig,
    UrlPropertyConfig,
    UniqueIdPropertyConfig,
    TitlePropertyConfig,
    RichTextPropertyConfig,
    CheckboxPropertyConfig,
    CreatedByPropertyConfig,
    CreatedTimePropertyConfig,
    LastEditedByPropertyConfig,
    LastEditedTimePropertyConfig,
    EmailPropertyConfig,
    FilesPropertyConfig,
    RelationPropertyConfig,
    RelationConfig,
    RelationDualConfig,
    RelationSingleConfig,
    RollupPropertyConfig,
    RollupSettings,
    FormulaPropertyConfig,
)
from .notion_client import NotionAsyncClient, NotionErrorCode
from .responses.list_response import (
    ListUsersResponse,
    QueryDatabaseResponse,
    QueryDataSourceResponse,
    SearchResponse,
    ListFileUploadsResponse,
)
from .helper import Field, NotionMapper, NotionPropertyDescriptor

__all__ = [
    # DataSource
    "DataSource",
    "PartialDataSource",
    # Database
    "NotionDatabase",
    "PartialDatabase",
    # Page
    "NotionPage",
    "PartialPage",
    "FileUploadObject",
    "PropertyItemObject",
    "PropertyItemListResponse",
    # List responses
    "ListUsersResponse",
    "QueryDatabaseResponse",
    "QueryDataSourceResponse",
    "SearchResponse",
    "ListFileUploadsResponse",
    # Block base
    "BlockObject",
    "PartialBlock",
    "BlockType",
    "ApiColor",
    "BaseBlockObject",
    # Text blocks
    "ParagraphBlock",
    "Heading1Block",
    "Heading2Block",
    "Heading3Block",
    "BulletedListItemBlock",
    "NumberedListItemBlock",
    "QuoteBlock",
    "ToDoBlock",
    "ToggleBlock",
    "TemplateBlock",
    # Special blocks
    "SyncedBlockBlock",
    "ChildPageBlock",
    "ChildDatabaseBlock",
    "EquationBlock",
    "CodeBlock",
    "CalloutBlock",
    # Layout blocks
    "DividerBlock",
    "BreadcrumbBlock",
    "TableOfContentsBlock",
    "ColumnListBlock",
    "ColumnBlock",
    "LinkToPageBlock",
    "TableBlock",
    "TableRowBlock",
    # Media blocks
    "EmbedBlock",
    "BookmarkBlock",
    "ImageBlock",
    "VideoBlock",
    "PdfBlock",
    "FileBlock",
    "AudioBlock",
    "LinkPreviewBlock",
    # Unsupported
    "UnsupportedBlock",
    # Requests
    "NumberPropertyRequest",
    "TitlePropertyRequest",
    "RichTextPropertyRequest",
    "CheckboxPropertyRequest",
    "SelectPropertyRequest",
    "MultiSelectPropertyRequest",
    "StatusPropertyRequest",
    "DatePropertyRequest",
    "PeoplePropertyRequest",
    "FilesPropertyRequest",
    "UrlPropertyRequest",
    "EmailPropertyRequest",
    "PhoneNumberPropertyRequest",
    "RelationPropertyRequest",
    "UpdatePageParameters",
    "CreatePageParameters",
    "RelationItemRequest",
    "DateRequest",
    "IdRequest",
    "SelectPropertyItemRequest",
    "PartialUserObjectRequest",
    "GroupObjectRequest",
    "AnnotationRequest",
    "PropertyRequest",
    # Filters
    # フィルター基底型
    "PropertyFilter",
    "TimestampFilter",
    "FilterCondition",
    "AndFilterDict",
    "OrFilterDict",
    "create_and_filter",
    "create_or_filter",
    # プロパティフィルター
    "PropertyFilterTitle",
    "PropertyFilterRichText",
    "PropertyFilterNumber",
    "PropertyFilterCheckbox",
    "PropertyFilterSelect",
    "PropertyFilterMultiSelect",
    "PropertyFilterStatus",
    "PropertyFilterDate",
    "PropertyFilterPeople",
    "PropertyFilterFiles",
    "PropertyFilterUrl",
    "PropertyFilterEmail",
    "PropertyFilterPhoneNumber",
    "PropertyFilterRelation",
    "PropertyFilterCreatedBy",
    "PropertyFilterCreatedTime",
    "PropertyFilterLastEditedBy",
    "PropertyFilterLastEditedTime",
    "PropertyFilterFormula",
    "PropertyFilterUniqueId",
    "PropertyFilterRollup",
    "PropertyFilterVerification",
    # 個別フィルター条件
    "TextPropertyFilter",
    "NumberPropertyFilter",
    "CheckboxPropertyFilter",
    "SelectPropertyFilter",
    "MultiSelectPropertyFilter",
    "StatusPropertyFilter",
    "DatePropertyFilter",
    "PeoplePropertyFilter",
    "RelationPropertyFilter",
    "FormulaPropertyFilter",
    "RollupPropertyFilter",
    "ExistencePropertyFilter",
    # 具体的なフィルター
    "TextFilterEquals",
    "TextFilterContains",
    "NumberFilterEquals",
    "NumberFilterGreaterThan",
    "DateFilterAfter",
    "DateFilterBefore",
    "RelationFilterContains",
    # Schema
    "PeoplePropertyConfig",
    "SelectPropertyConfig",
    "MultiSelectPropertyConfig",
    "MultiSelectOptions",
    "StatusPropertyConfig",
    "SelectOptionConfig",
    "SelectOptions",
    "DatePropertyConfig",
    "NumberPropertyConfig",
    "PhoneNumberPropertyConfig",
    "UrlPropertyConfig",
    "UniqueIdPropertyConfig",
    "TitlePropertyConfig",
    "RichTextPropertyConfig",
    "CheckboxPropertyConfig",
    "CreatedByPropertyConfig",
    "CreatedTimePropertyConfig",
    "LastEditedByPropertyConfig",
    "LastEditedTimePropertyConfig",
    "EmailPropertyConfig",
    "FilesPropertyConfig",
    "RelationPropertyConfig",
    "RelationConfig",
    "RelationDualConfig",
    "RelationSingleConfig",
    "RollupPropertyConfig",
    "RollupSettings",
    "FormulaPropertyConfig",
    # Client
    "NotionAsyncClient",
    "NotionErrorCode",
    # Helper
    "Field",
    "NotionMapper",
    "NotionPropertyDescriptor",
]



"""
複合フィルター定義

AND/ORなどの複合条件フィルター。
TypedDict を使用して公式 Notion API 仕様に準拠。

公式TypeScript定義:
type PropertyOrTimestampFilter = PropertyFilter | TimestampFilter
type PropertyOrTimestampFilterArray = Array<PropertyOrTimestampFilter>
type GroupFilterOperatorArray = Array<
  | PropertyOrTimestampFilter
  | { or: PropertyOrTimestampFilterArray }
  | { and: PropertyOrTimestampFilterArray }
>
"""

from __future__ import annotations

from typing import TYPE_CHECKING, Any, Union

if TYPE_CHECKING:
    from typing import TypedDict

    from .property_filters import PropertyFilter
    from .timestamp_filters import TimestampFilter

    # TYPE_CHECKING時のみ使用: 正確な型定義
    PropertyOrTimestampFilter = Union[PropertyFilter, TimestampFilter]

    class AndFilterDict(TypedDict):
        """ANDフィルター (型チェック用)"""

        and_: list[Union[PropertyOrTimestampFilter, "AndFilterDict", "OrFilterDict"]]  # type: ignore

    class OrFilterDict(TypedDict):
        """ORフィルター (型チェック用)"""

        or_: list[Union[PropertyOrTimestampFilter, "AndFilterDict", "OrFilterDict"]]  # type: ignore

    # 完全な FilterCondition 型定義
    FilterCondition = Union[
        PropertyFilter,
        TimestampFilter,
        AndFilterDict,
        OrFilterDict,
    ]
else:
    # 実行時: 循環参照を避けるため dict を使用
    AndFilterDict = dict[str, list[Any]]  # {"and": [...]}
    OrFilterDict = dict[str, list[Any]]  # {"or": [...]}
    FilterCondition = dict[str, Any]


# ============================================================================
# Helper Functions for Type-Safe Filter Construction
# ============================================================================

if TYPE_CHECKING:
    # 型チェック時: 正確な型シグネチャ
    def create_and_filter(
        *conditions: Union[PropertyFilter, TimestampFilter, AndFilterDict, OrFilterDict]
    ) -> AndFilterDict: ...

    def create_or_filter(
        *conditions: Union[PropertyFilter, TimestampFilter, AndFilterDict, OrFilterDict]
    ) -> OrFilterDict: ...

else:
    # 実行時: 実装
    def create_and_filter(*conditions: dict[str, Any]) -> AndFilterDict:
        """AND フィルターを作成するヘルパー関数

        すべての条件を満たすレコードを抽出する。

        Examples:
            ```python
            from notion_py_client.filters import create_and_filter

            filter = create_and_filter(
                {"property": "Status", "status": {"equals": "Active"}},
                {"property": "Amount", "number": {"greater_than": 10000}},
            )
            # => {"and": [{...}, {...}]}
            ```

        Args:
            *conditions: フィルター条件（PropertyFilter, TimestampFilter, または入れ子の複合フィルター）

        Returns:
            ANDフィルター辞書 `{"and": [...]}`
        """
        return {"and": list(conditions)}

    def create_or_filter(*conditions: dict[str, Any]) -> OrFilterDict:
        """OR フィルターを作成するヘルパー関数

        いずれかの条件を満たすレコードを抽出する。

        Examples:
            ```python
            from notion_py_client.filters import create_or_filter

            filter = create_or_filter(
                {"property": "Service", "select": {"equals": "A"}},
                {"property": "Service", "select": {"equals": "B"}},
            )
            # => {"or": [{...}, {...}]}
            ```

        Args:
            *conditions: フィルター条件（PropertyFilter, TimestampFilter, または入れ子の複合フィルター）

        Returns:
            ORフィルター辞書 `{"or": [...]}`
        """
        return {"or": list(conditions)}



from typing import Literal

from pydantic import Field

from ...models import PartialUser, User
from ._base_property import BaseProperty, NotionPropertyType


class CreatedByProperty(BaseProperty[Literal[NotionPropertyType.CREATED_BY]]):
    """Notionのcreated_byプロパティ"""

    type: Literal[NotionPropertyType.CREATED_BY] = Field(
        NotionPropertyType.CREATED_BY, description="プロパティタイプ"
    )

    created_by: PartialUser | User = Field(..., description="作成者情報")

    def get_display_value(self) -> str | int | float | bool | None:
        """作成者情報を取得

        Returns:
            str | None: 作成者名。作成者情報が不完全な場合はNone
        """
        name = getattr(self.created_by, "name", None)
        return name if name else None



"""
Notionブロックモデル

ブロックタイプごとにファイルを分割して実装
"""

from .base import ApiColor, BlockType, BaseBlockObject, PartialBlock
from .text_blocks import (
    ParagraphBlock,
    Heading1Block,
    Heading2Block,
    Heading3Block,
    BulletedListItemBlock,
    NumberedListItemBlock,
    QuoteBlock,
    ToDoBlock,
    ToggleBlock,
    TemplateBlock,
)
from .special_blocks import (
    SyncedBlockBlock,
    ChildPageBlock,
    ChildDatabaseBlock,
    EquationBlock,
    CodeBlock,
    CalloutBlock,
)
from .layout_blocks import (
    DividerBlock,
    BreadcrumbBlock,
    TableOfContentsBlock,
    ColumnListBlock,
    ColumnBlock,
    LinkToPageBlock,
    TableBlock,
    TableRowBlock,
)
from .media_blocks import (
    EmbedBlock,
    BookmarkBlock,
    ImageBlock,
    VideoBlock,
    PdfBlock,
    FileBlock,
    AudioBlock,
    LinkPreviewBlock,
)
from .unsupported import UnsupportedBlock

# Union type for all block types
BlockObject = (
    ParagraphBlock
    | Heading1Block
    | Heading2Block
    | Heading3Block
    | BulletedListItemBlock
    | NumberedListItemBlock
    | QuoteBlock
    | ToDoBlock
    | ToggleBlock
    | TemplateBlock
    | SyncedBlockBlock
    | ChildPageBlock
    | ChildDatabaseBlock
    | EquationBlock
    | CodeBlock
    | CalloutBlock
    | DividerBlock
    | BreadcrumbBlock
    | TableOfContentsBlock
    | ColumnListBlock
    | ColumnBlock
    | LinkToPageBlock
    | TableBlock
    | TableRowBlock
    | EmbedBlock
    | BookmarkBlock
    | ImageBlock
    | VideoBlock
    | PdfBlock
    | FileBlock
    | AudioBlock
    | LinkPreviewBlock
    | UnsupportedBlock
)

__all__ = [
    # Base
    "ApiColor",
    "BlockType",
    "BaseBlockObject",
    "PartialBlock",
    "BlockObject",
    # Text blocks
    "ParagraphBlock",
    "Heading1Block",
    "Heading2Block",
    "Heading3Block",
    "BulletedListItemBlock",
    "NumberedListItemBlock",
    "QuoteBlock",
    "ToDoBlock",
    "ToggleBlock",
    "TemplateBlock",
    # Special blocks
    "SyncedBlockBlock",
    "ChildPageBlock",
    "ChildDatabaseBlock",
    "EquationBlock",
    "CodeBlock",
    "CalloutBlock",
    # Layout blocks
    "DividerBlock",
    "BreadcrumbBlock",
    "TableOfContentsBlock",
    "ColumnListBlock",
    "ColumnBlock",
    "LinkToPageBlock",
    "TableBlock",
    "TableRowBlock",
    # Media blocks
    "EmbedBlock",
    "BookmarkBlock",
    "ImageBlock",
    "VideoBlock",
    "PdfBlock",
    "FileBlock",
    "AudioBlock",
    "LinkPreviewBlock",
    # Unsupported
    "UnsupportedBlock",
]



from typing import Literal

from pydantic import Field

from ._base_config import BasePropertyConfig
from ....models.primitives import EmptyObject
from ....properties.base_properties._base_property import NotionPropertyType


class LastVisitedTimePropertyConfig(
    BasePropertyConfig[Literal[NotionPropertyType.LAST_VISITED_TIME]]
):
    """Notionのlast_visited_timeプロパティ設定

    TypeScript: LastVisitedTimePropertyConfigurationRequest
    フィールドは空オブジェクトで構成されます。
    """

    type: Literal[NotionPropertyType.LAST_VISITED_TIME] = Field(
        NotionPropertyType.LAST_VISITED_TIME, description="プロパティタイプ"
    )
    last_visited_time: EmptyObject = Field(
        default_factory=EmptyObject, description="last_visited_time設定"
    )




from typing import Literal

from pydantic import BaseModel, Field, StrictStr


class InternalFile(BaseModel):
    """Notionの内部ファイル"""

    url: StrictStr = Field(..., description="ファイルURL")
    expiry_time: StrictStr = Field(..., description="URLの有効期限")


class ExternalFile(BaseModel):
    """Notionの外部ファイル"""

    url: StrictStr = Field(..., description="外部リソースURL")


class FileWithName(BaseModel):
    """Notionのファイル（名前付き）"""

    name: StrictStr = Field(..., description="ファイル名")
    type: Literal["file", "external"] = Field(
        ..., description="ファイルタイプ"
    )
    file: InternalFile | None = Field(
        None, description="内部ファイル（typeがfileの場合のみ）"
    )
    external: ExternalFile | None = Field(
        None, description="外部ファイル（typeがexternalの場合のみ）"
    )


class FileObject(BaseModel):
    """Notionのfilesプロパティが保持するファイルオブジェクト"""

    files: list[FileWithName] = Field(
        default_factory=list, description="ファイルのリスト"
    )

    def get_urls(self) -> list[str]:
        """ファイルに紐づくURLの一覧を取得"""
        urls: list[str] = []
        for file_item in self.files:
            if file_item.type == "file" and file_item.file:
                urls.append(file_item.file.url)
            elif file_item.type == "external" and file_item.external:
                urls.append(file_item.external.url)
        return urls



"""
プロパティフィルター統合型

各プロパティタイプに対応するフィルタークラス。
TypedDict を使用して公式 Notion API 仕様に準拠。

公式TypeScript定義:
type PropertyFilter =
  | { title: TextPropertyFilter; property: string; type?: "title" }
  | { rich_text: TextPropertyFilter; property: string; type?: "rich_text" }
  | { number: NumberPropertyFilter; property: string; type?: "number" }
  | { checkbox: CheckboxPropertyFilter; property: string; type?: "checkbox" }
  | { select: SelectPropertyFilter; property: string; type?: "select" }
  | { multi_select: MultiSelectPropertyFilter; property: string; type?: "multi_select" }
  | { status: StatusPropertyFilter; property: string; type?: "status" }
  | { date: DatePropertyFilter; property: string; type?: "date" }
  | { people: PeoplePropertyFilter; property: string; type?: "people" }
  | { files: ExistencePropertyFilter; property: string; type?: "files" }
  | { url: TextPropertyFilter; property: string; type?: "url" }
  | { email: TextPropertyFilter; property: string; type?: "email" }
  | { phone_number: TextPropertyFilter; property: string; type?: "phone_number" }
  | { relation: RelationPropertyFilter; property: string; type?: "relation" }
  | { created_by: PeoplePropertyFilter; property: string; type?: "created_by" }
  | { created_time: DatePropertyFilter; property: string; type?: "created_time" }
  | { last_edited_by: PeoplePropertyFilter; property: string; type?: "last_edited_by" }
  | { last_edited_time: DatePropertyFilter; property: string; type?: "last_edited_time" }
  | { formula: FormulaPropertyFilter; property: string; type?: "formula" }
  | { unique_id: NumberPropertyFilter; property: string; type?: "unique_id" }
  | { rollup: RollupPropertyFilter; property: string; type?: "rollup" }
  | { verification: VerificationPropertyStatusFilter; property: string; type?: "verification" }
"""

from typing import Literal, NotRequired, TypedDict, Union

from .advanced_filters import (
    FormulaPropertyFilter,
    RollupPropertyFilter,
    VerificationPropertyStatusFilter,
)
from .base_filters import ExistencePropertyFilter
from .date_filters import DatePropertyFilter
from .relation_filters import PeoplePropertyFilter, RelationPropertyFilter
from .value_filters import (
    CheckboxPropertyFilter,
    MultiSelectPropertyFilter,
    NumberPropertyFilter,
    SelectPropertyFilter,
    StatusPropertyFilter,
    TextPropertyFilter,
)


# ============================================================================
# Property Filter TypedDicts
# ============================================================================


class PropertyFilterTitle(TypedDict):
    """タイトルプロパティフィルター

    Examples:
        ```python
        filter: PropertyFilterTitle = {
            "property": "Name",
            "title": {"contains": "Important"}
        }
        ```
    """

    property: str
    title: TextPropertyFilter
    type: NotRequired[Literal["title"]]


class PropertyFilterRichText(TypedDict):
    """リッチテキストプロパティフィルター"""

    property: str
    rich_text: TextPropertyFilter
    type: NotRequired[Literal["rich_text"]]


class PropertyFilterNumber(TypedDict):
    """数値プロパティフィルター"""

    property: str
    number: NumberPropertyFilter
    type: NotRequired[Literal["number"]]


class PropertyFilterCheckbox(TypedDict):
    """チェックボックスプロパティフィルター"""

    property: str
    checkbox: CheckboxPropertyFilter
    type: NotRequired[Literal["checkbox"]]


class PropertyFilterSelect(TypedDict):
    """セレクトプロパティフィルター"""

    property: str
    select: SelectPropertyFilter
    type: NotRequired[Literal["select"]]


class PropertyFilterMultiSelect(TypedDict):
    """マルチセレクトプロパティフィルター"""

    property: str
    multi_select: MultiSelectPropertyFilter
    type: NotRequired[Literal["multi_select"]]


class PropertyFilterStatus(TypedDict):
    """ステータスプロパティフィルター"""

    property: str
    status: StatusPropertyFilter
    type: NotRequired[Literal["status"]]


class PropertyFilterDate(TypedDict):
    """日付プロパティフィルター"""

    property: str
    date: DatePropertyFilter
    type: NotRequired[Literal["date"]]


class PropertyFilterPeople(TypedDict):
    """Peopleプロパティフィルター"""

    property: str
    people: PeoplePropertyFilter
    type: NotRequired[Literal["people"]]


class PropertyFilterFiles(TypedDict):
    """ファイルプロパティフィルター"""

    property: str
    files: ExistencePropertyFilter
    type: NotRequired[Literal["files"]]


class PropertyFilterUrl(TypedDict):
    """URLプロパティフィルター"""

    property: str
    url: TextPropertyFilter
    type: NotRequired[Literal["url"]]


class PropertyFilterEmail(TypedDict):
    """メールプロパティフィルター"""

    property: str
    email: TextPropertyFilter
    type: NotRequired[Literal["email"]]


class PropertyFilterPhoneNumber(TypedDict):
    """電話番号プロパティフィルター"""

    property: str
    phone_number: TextPropertyFilter
    type: NotRequired[Literal["phone_number"]]


class PropertyFilterRelation(TypedDict):
    """Relationプロパティフィルター"""

    property: str
    relation: RelationPropertyFilter
    type: NotRequired[Literal["relation"]]


class PropertyFilterCreatedBy(TypedDict):
    """作成者プロパティフィルター"""

    property: str
    created_by: PeoplePropertyFilter
    type: NotRequired[Literal["created_by"]]


class PropertyFilterCreatedTime(TypedDict):
    """作成日時プロパティフィルター"""

    property: str
    created_time: DatePropertyFilter
    type: NotRequired[Literal["created_time"]]


class PropertyFilterLastEditedBy(TypedDict):
    """最終編集者プロパティフィルター"""

    property: str
    last_edited_by: PeoplePropertyFilter
    type: NotRequired[Literal["last_edited_by"]]


class PropertyFilterLastEditedTime(TypedDict):
    """最終編集日時プロパティフィルター"""

    property: str
    last_edited_time: DatePropertyFilter
    type: NotRequired[Literal["last_edited_time"]]


class PropertyFilterFormula(TypedDict):
    """Formulaプロパティフィルター"""

    property: str
    formula: FormulaPropertyFilter
    type: NotRequired[Literal["formula"]]


class PropertyFilterUniqueId(TypedDict):
    """ユニークIDプロパティフィルター"""

    property: str
    unique_id: NumberPropertyFilter
    type: NotRequired[Literal["unique_id"]]


class PropertyFilterRollup(TypedDict):
    """Rollupプロパティフィルター"""

    property: str
    rollup: RollupPropertyFilter
    type: NotRequired[Literal["rollup"]]


class PropertyFilterVerification(TypedDict):
    """検証プロパティフィルター"""

    property: str
    verification: VerificationPropertyStatusFilter
    type: NotRequired[Literal["verification"]]


# ============================================================================
# Property Filter Union Type
# ============================================================================

# Matches TypeScript: type PropertyFilter = ...
PropertyFilter = Union[
    PropertyFilterTitle,
    PropertyFilterRichText,
    PropertyFilterNumber,
    PropertyFilterCheckbox,
    PropertyFilterSelect,
    PropertyFilterMultiSelect,
    PropertyFilterStatus,
    PropertyFilterDate,
    PropertyFilterPeople,
    PropertyFilterFiles,
    PropertyFilterUrl,
    PropertyFilterEmail,
    PropertyFilterPhoneNumber,
    PropertyFilterRelation,
    PropertyFilterCreatedBy,
    PropertyFilterCreatedTime,
    PropertyFilterLastEditedBy,
    PropertyFilterLastEditedTime,
    PropertyFilterFormula,
    PropertyFilterUniqueId,
    PropertyFilterRollup,
    PropertyFilterVerification,
]



"""
Notion親オブジェクトモデル

Database、Page、DataSource、Blockなどで使用される親オブジェクトの定義
"""

from __future__ import annotations

from typing import Literal, TypedDict, NotRequired


class DatabaseParent(TypedDict):
    """データベースを親とするオブジェクト.

    Examples:
        >>> parent: DatabaseParent = {
        ...     "type": "database_id",
        ...     "database_id": "abc123"
        ... }
    """

    type: Literal["database_id"]
    database_id: str


class PageParent(TypedDict):
    """ページを親とするオブジェクト.

    Examples:
        >>> parent: PageParent = {
        ...     "type": "page_id",
        ...     "page_id": "abc123"
        ... }
    """

    type: Literal["page_id"]
    page_id: str


class WorkspaceParent(TypedDict):
    """ワークスペースを親とするオブジェクト.

    Examples:
        >>> parent: WorkspaceParent = {
        ...     "type": "workspace",
        ...     "workspace": True
        ... }
    """

    type: Literal["workspace"]
    workspace: Literal[True]


class BlockParent(TypedDict):
    """ブロックを親とするオブジェクト.

    Examples:
        >>> parent: BlockParent = {
        ...     "type": "block_id",
        ...     "block_id": "abc123"
        ... }
    """

    type: Literal["block_id"]
    block_id: str


class DataSourceParent(TypedDict):
    """データソースを親とするオブジェクト.

    Examples:
        >>> parent: DataSourceParent = {
        ...     "type": "data_source_id",
        ...     "data_source_id": "abc123"
        ... }
    """

    type: Literal["data_source_id"]
    data_source_id: str


# すべての親タイプのUnion型
NotionParent = (
    DatabaseParent | PageParent | WorkspaceParent | BlockParent | DataSourceParent
)



"""Unit tests for PropertyItem response types."""

import pytest
from pydantic import TypeAdapter
from notion_py_client.responses.property_item import (
    PropertyItemObject,
    PropertyItemListResponse,
    NumberPropertyItem,
    UrlPropertyItem,
    SelectPropertyItem,
    StatusPropertyItem,
    DatePropertyItem,
    EmailPropertyItem,
    PhoneNumberPropertyItem,
    CheckboxPropertyItem,
    TitlePropertyItem,
    RichTextPropertyItem,
    PeoplePropertyItem,
)


class TestNumberPropertyItem:
    """Test NumberPropertyItem."""

    def test_parse_number_property_item(self):
        """Test parsing a number property item from API response."""
        data = {
            "object": "property_item",
            "id": "prop_123",
            "type": "number",
            "number": 42.5,
        }

        adapter = TypeAdapter(PropertyItemObject)
        result = adapter.validate_python(data)

        assert isinstance(result, NumberPropertyItem)
        assert result.type == "number"
        assert result.number == 42.5
        assert result.id == "prop_123"

    def test_parse_null_number_property_item(self):
        """Test parsing a number property item with null value."""
        data = {
            "object": "property_item",
            "id": "prop_456",
            "type": "number",
            "number": None,
        }

        adapter = TypeAdapter(PropertyItemObject)
        result = adapter.validate_python(data)

        assert isinstance(result, NumberPropertyItem)
        assert result.number is None


class TestUrlPropertyItem:
    """Test UrlPropertyItem."""

    def test_parse_url_property_item(self):
        """Test parsing a URL property item."""
        data = {
            "object": "property_item",
            "id": "prop_url",
            "type": "url",
            "url": "https://example.com",
        }

        adapter = TypeAdapter(PropertyItemObject)
        result = adapter.validate_python(data)

        assert isinstance(result, UrlPropertyItem)
        assert result.type == "url"
        assert result.url == "https://example.com"


class TestSelectPropertyItem:
    """Test SelectPropertyItem."""

    def test_parse_select_property_item(self):
        """Test parsing a select property item."""
        data = {
            "object": "property_item",
            "id": "prop_select",
            "type": "select",
            "select": {
                "id": "option_1",
                "name": "Option A",
                "color": "blue",
            },
        }

        adapter = TypeAdapter(PropertyItemObject)
        result = adapter.validate_python(data)

        assert isinstance(result, SelectPropertyItem)
        if result.select is not None:
            assert result.select.id == "option_1"
            assert result.select.name == "Option A"
            assert result.select.color == "blue"



class TestStatusPropertyItem:
    """Test StatusPropertyItem."""

    def test_parse_status_property_item(self):
        """Test parsing a status property item."""
        data = {
            "object": "property_item",
            "id": "prop_status",
            "type": "status",
            "status": {
                "id": "status_1",
                "name": "In Progress",
                "color": "yellow",
            },
        }

        adapter = TypeAdapter(PropertyItemObject)
        result = adapter.validate_python(data)

        assert isinstance(result, StatusPropertyItem)
        assert result.status is not None
        assert result.status.name == "In Progress"


class TestDatePropertyItem:
    """Test DatePropertyItem."""

    def test_parse_date_property_item(self):
        """Test parsing a date property item."""
        data = {
            "object": "property_item",
            "id": "prop_date",
            "type": "date",
            "date": {
                "start": "2025-01-01",
                "end": "2025-01-31",
                "time_zone": "Asia/Tokyo",
            },
        }

        adapter = TypeAdapter(PropertyItemObject)
        result = adapter.validate_python(data)

        assert isinstance(result, DatePropertyItem)
        assert result.date is not None
        assert result.date.start == "2025-01-01"
        assert result.date.end == "2025-01-31"
        assert result.date.time_zone == "Asia/Tokyo"


class TestEmailPropertyItem:
    """Test EmailPropertyItem."""

    def test_parse_email_property_item(self):
        """Test parsing an email property item."""
        data = {
            "object": "property_item",
            "id": "prop_email",
            "type": "email",
            "email": "test@example.com",
        }

        adapter = TypeAdapter(PropertyItemObject)
        result = adapter.validate_python(data)

        assert isinstance(result, EmailPropertyItem)
        assert result.email == "test@example.com"


class TestPhoneNumberPropertyItem:
    """Test PhoneNumberPropertyItem."""

    def test_parse_phone_number_property_item(self):
        """Test parsing a phone number property item."""
        data = {
            "object": "property_item",
            "id": "prop_phone",
            "type": "phone_number",
            "phone_number": "+81-90-1234-5678",
        }

        adapter = TypeAdapter(PropertyItemObject)
        result = adapter.validate_python(data)

        assert isinstance(result, PhoneNumberPropertyItem)
        assert result.phone_number == "+81-90-1234-5678"


class TestCheckboxPropertyItem:
    """Test CheckboxPropertyItem."""

    def test_parse_checkbox_property_item_true(self):
        """Test parsing a checkbox property item (checked)."""
        data = {
            "object": "property_item",
            "id": "prop_checkbox",
            "type": "checkbox",
            "checkbox": True,
        }

        adapter = TypeAdapter(PropertyItemObject)
        result = adapter.validate_python(data)

        assert isinstance(result, CheckboxPropertyItem)
        assert result.checkbox is True

    def test_parse_checkbox_property_item_false(self):
        """Test parsing a checkbox property item (unchecked)."""
        data = {
            "object": "property_item",
            "id": "prop_checkbox2",
            "type": "checkbox",
            "checkbox": False,
        }

        adapter = TypeAdapter(PropertyItemObject)
        result = adapter.validate_python(data)

        assert isinstance(result, CheckboxPropertyItem)
        assert result.checkbox is False


class TestTitlePropertyItem:
    """Test TitlePropertyItem."""

    def test_parse_title_property_item(self):
        """Test parsing a title property item."""
        data = {
            "object": "property_item",
            "id": "prop_title",
            "type": "title",
            "title": {
                "type": "text",
                "text": {"content": "Page Title", "link": None},
                "plain_text": "Page Title",
                "href": None,
                "annotations": {
                    "bold": False,
                    "italic": False,
                    "strikethrough": False,
                    "underline": False,
                    "code": False,
                    "color": "default",
                },
            },
        }

        adapter = TypeAdapter(PropertyItemObject)
        result = adapter.validate_python(data)

        assert isinstance(result, TitlePropertyItem)
        assert result.title.plain_text == "Page Title"


class TestRichTextPropertyItem:
    """Test RichTextPropertyItem."""

    def test_parse_rich_text_property_item(self):
        """Test parsing a rich text property item."""
        data = {
            "object": "property_item",
            "id": "prop_richtext",
            "type": "rich_text",
            "rich_text": {
                "type": "text",
                "text": {"content": "Rich content", "link": None},
                "plain_text": "Rich content",
                "href": None,
                "annotations": {
                    "bold": True,
                    "italic": False,
                    "strikethrough": False,
                    "underline": False,
                    "code": False,
                    "color": "default",
                },
            },
        }

        adapter = TypeAdapter(PropertyItemObject)
        result = adapter.validate_python(data)

        assert isinstance(result, RichTextPropertyItem)
        assert result.rich_text.plain_text == "Rich content"
        assert result.rich_text.annotations.bold is True


class TestPeoplePropertyItem:
    """Test PeoplePropertyItem."""

    def test_parse_people_property_item(self):
        """Test parsing a people property item."""
        data = {
            "object": "property_item",
            "id": "prop_people",
            "type": "people",
            "people": {
                "object": "user",
                "id": "user_123",
            },
        }

        adapter = TypeAdapter(PropertyItemObject)
        result = adapter.validate_python(data)

        assert isinstance(result, PeoplePropertyItem)
        assert result.people.id == "user_123"


class TestPropertyItemListResponse:
    """Test PropertyItemListResponse."""

    def test_parse_property_item_list_response(self):
        """Test parsing a paginated property item list."""
        data = {
            "object": "list",
            "next_cursor": "cursor_abc",
            "has_more": True,
            "type": "property_item",
            "property_item": {
                "type": "property_item",
                "next_url": "https://example.com/next",
                "id": "prop_123",
            },
            "results": [
                {
                    "object": "property_item",
                    "id": "item_1",
                    "type": "number",
                    "number": 10,
                },
                {
                    "object": "property_item",
                    "id": "item_2",
                    "type": "number",
                    "number": 20,
                },
            ],
        }

        result = PropertyItemListResponse.model_validate(data)

        assert result.object == "list"
        assert result.has_more is True
        assert result.next_cursor == "cursor_abc"
        assert len(result.results) == 2
        assert isinstance(result.results[0], NumberPropertyItem)
        assert isinstance(result.results[1], NumberPropertyItem)
        if isinstance(result.results[0], NumberPropertyItem):
            assert result.results[0].number == 10
        if isinstance(result.results[1], NumberPropertyItem):
            assert result.results[1].number == 20
